SET TRANSACTION READ WRITE;

DROP SEQUENCE SEQ_ID_LOGS;

DROP TABLE TB_ALIMENTACION;
DROP TABLE TB_DISTRIBUCION;
DROP TABLE TB_LECTURAS;
DROP TABLE TB_CENTRALES_D;
DROP TABLE TB_LINEAS;
DROP TABLE TB_HISTORICO;
DROP TABLE TB_CENTRALES_P;
DROP TABLE TB_CONTRATOS;
DROP TABLE TB_CONTADORES;
DROP TABLE TB_CLIENTES;
DROP TABLE TB_LOGS;

CREATE TABLE tb_Clientes
(
  id_cliente VARCHAR2(15) NOT NULL,
  nombre VARCHAR(50)NOT NULL,
  direccion VARCHAR2(50)NOT NULL,
  localidad VARCHAR2(50)NOT NULL,
  pais VARCHAR(25)NOT NULL,
  CONSTRAINT pk_clientes PRIMARY KEY(id_cliente)
);

CREATE TABLE tb_Contadores
(
  id_contador NUMBER(10) NOT NULL,
  direccion VARCHAR2(50) NOT NULL,
  localidad VARCHAR2(50) NOT NULL,
  pais VARCHAR2(25) NOT NULL,
  modelo VARCHAR2(15) NOT NULL,
  any_fabricacion NUMBER(4) NOT NULL,
  fabricante VARCHAR2(25) NOT NULL,
  consumo_acumulado NUMBER(10,2) NOT NULL,
  CONSTRAINT pk_contadores PRIMARY KEY(id_contador)
);

CREATE TABLE tb_Contratos
(
  id_contrato VARCHAR2(15) NOT NULL,
  fecha_alta DATE NOT NULL,
  fecha_baja DATE,
  potencia_contratada NUMBER(5,2) NOT NULL,
  cliente VARCHAR2(15)NOT NULL,
  contador NUMBER(10) NOT NULL,
  CONSTRAINT pk_contratos PRIMARY KEY(id_contrato),
  CONSTRAINT fk_clientes FOREIGN KEY(cliente)REFERENCES tb_Clientes(id_cliente),
  CONSTRAINT fk_contador_contrato FOREIGN KEY(contador)REFERENCES tb_Contadores(id_contador),
  CONSTRAINT ck_fecha_baja CHECK (fecha_baja > fecha_alta)
);

CREATE TABLE tb_Centrales_P
(
  codigo_central VARCHAR2(10) NOT NULL,
  tipo_central VARCHAR2(10) NOT NULL,
  direccion VARCHAR2(50)NOT NULL,
  localidad VARCHAR2(50)NOT NULL,
  pais VARCHAR2(25)NOT NULL,
  energia_funcionamiento NUMBER,
  kg_residuos_radiactivos NUMBER,
  kg_CO2 NUMBER,
  numero_molinos NUMBER,
  numero_paneles NUMBER,
  fecha_ultima_inspeccion DATE NOT NULL,
  maxima_produccion_mes NUMBER(16) DEFAULT (0),
  CONSTRAINT pk_centrales_P PRIMARY KEY (codigo_central),
  CONSTRAINT ck_tipo_central CHECK (tipo_central IN('NUCLEAR', 'TERMICA','CARBON','EOLICA','SOLAR'))
);

CREATE TABLE tb_Historico
(
  central_P VARCHAR2(10) NOT NULL,
  mes NUMBER(2) NOT NULL,
  anyo NUMBER(4) NOT NULL,
  produccion NUMBER(16) DEFAULT (0),
  CONSTRAINT pk_produccion_mes PRIMARY KEY (central_P, mes, anyo)
);

CREATE TABLE tb_Lineas
(
  codigo_linea NUMBER(10) NOT NULL,
  capacidad NUMBER(8,2) NOT NULL,
  central_p VARCHAR2(10) NOT NULL,
  CONSTRAINT pk_lineas PRIMARY KEY (codigo_linea),
  CONSTRAINT fk_central_p_linea FOREIGN KEY(central_p)REFERENCES tb_Centrales_P(codigo_central)
);

CREATE TABLE tb_Centrales_D
(
  direccion VARCHAR2(50)NOT NULL,
  localidad VARCHAR2(50)NOT NULL,
  pais VARCHAR(25)NOT NULL,
  suministro_maximo NUMBER(8,2) NOT NULL,
  energia_emitida NUMBER(10,2)DEFAULT (0),
  CONSTRAINT pk_centrales_D PRIMARY KEY(direccion,localidad,pais)
);

CREATE TABLE tb_Lecturas
(
  contador NUMBER(10) NOT NULL,
  fecha DATE NOT NULL,
  modalidad VARCHAR2(15)NOT NULL,
  lectura_total NUMBER(10,2) NOT NULL,
  consumo_mes NUMBER(6,2),
  CONSTRAINT pk_lecturas PRIMARY KEY(contador,fecha),
  CONSTRAINT fk_contador_lecturas FOREIGN KEY(contador)REFERENCES tb_Contadores(id_contador)
);

CREATE TABLE tb_Distribucion
(
  contador NUMBER(10) NOT NULL,
  direccion VARCHAR2(50)NOT NULL,
  localidad VARCHAR2(50)NOT NULL,
  pais VARCHAR(25)NOT NULL,
  CONSTRAINT pk_distribucion PRIMARY KEY (contador,direccion,localidad,pais),
  CONSTRAINT fk_contadores_d_distribucion FOREIGN KEY (contador)REFERENCES tb_Contadores(id_contador),
  CONSTRAINT fk_centrales_d FOREIGN KEY (direccion, localidad, pais)REFERENCES tb_Centrales_D(direccion,localidad,pais)
  
);
  
CREATE TABLE tb_Alimentacion
(
  direccion VARCHAR2(50)NOT NULL,
  localidad VARCHAR2(50)NOT NULL,
  pais VARCHAR(25)NOT NULL,
  id_linea NUMBER(10)NOT NULL,
  CONSTRAINT pk_alimentacion PRIMARY KEY (direccion,localidad,pais,id_linea),
  CONSTRAINT fk_centrales_d_alimentacion FOREIGN KEY (direccion, localidad, pais)REFERENCES tb_Centrales_D(direccion,localidad,pais),
  CONSTRAINT fk_lineas_de_alimentacion FOREIGN KEY (id_linea)REFERENCES tb_Lineas(codigo_linea)
);
      
CREATE TABLE tb_Logs
(
  id_log NUMBER NOT NULL,
  fecha_proceso VARCHAR2(20)NOT NULL,
  nombre_procedimiento VARCHAR2(30)NOT NULL,
  parametros_in CLOB,
  parametros_out CLOB,
  parametro_RSP CLOB NOT NULL,
  CONSTRAINT pk_logs PRIMARY KEY (id_log)
);

CREATE SEQUENCE seq_id_logs 
CACHE 10;
 
COMMIT;
