--Alta Cliente--
/*Procedimiento que a partir de los parmetros de entrada aade un nuevo cliente a la base de datos
 *muestra el resultado por pantalla.
 *PARAM IN: identificacion(Variable de tipo Varchar2, contiene el NIF, CIF o cualquier otro identificador nico para el cliente).
 *	    nombre (Variable de tipo Varchar2, contiene el nombre y los apellidos del cliente o la Razn social en caso de empresas)
 *	    direccin (Variable de tipo Varchar2, contiene la direccin completa del cliente -calle, nmero, piso-)
 *	    localidad (Variable de tipo Varchar2, contiene el nombre de la localidad correspondiente al domicilio del cliente)
 *	    pas (Variable de tipo Varchar2, contiene el pas del domiciolio del cliente)
 *	    RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno)
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE PROCEDURE pr_alta_cliente(codigo IN VARCHAR2, nom IN VARCHAR2, dir IN VARCHAR2,loc IN VARCHAR2, ps IN VARCHAR2,  RSP OUT NOCOPY CLOB) AS
   
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(15);
    In_log CLOB;
    existe INTEGER;
    
    --Declaracin de excepciones
    EX_CLIENTE_EXISTE EXCEPTION;
    EX_CLIENTE_NOMBRE_NULL EXCEPTION;
    EX_CLIENTE_DIRECCION_NULL EXCEPTION;
    EX_CLIENTE_LOCALIDAD_NULL EXCEPTION;
    EX_CLIENTE_PAIS_NULL EXCEPTION;
  
    --Ejecucin del procedimiento.
    BEGIN
          
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_cliente';
      
        --Creamos una cadena de texto con los parametros de entrada separados por barras.
        In_log:= (codigo||'  ||  ' || nom||'  ||  ' || dir||'  ||  ' || loc||'  ||  ' || ps);        
        --Comprobamos que no existe un cliente con la misma identificacin lo cual
        --supondra una violacin de la PK.
        SELECT COUNT(*)INTO existe FROM tb_Clientes c WHERE c.id_cliente=codigo;
        
        --Si existe una violacin de la PK capturamos la excepcin para su posterior tratamiento.
        IF(existe > 0) THEN RAISE EX_CLIENTE_EXISTE; END IF;
        --Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(nom is NULL)THEN RAISE EX_CLIENTE_NOMBRE_NULL; END IF;
        IF(dir is NULL)THEN RAISE EX_CLIENTE_DIRECCION_NULL; END IF;
        IF(loc is NULL)THEN RAISE EX_CLIENTE_LOCALIDAD_NULL; END IF;
        IF(ps is NULL)THEN RAISE EX_CLIENTE_PAIS_NULL; END IF;
    
        INSERT INTO tb_Clientes VALUES(codigo,nom,dir,loc,ps);
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos commit para confirmar la insercin definitiva en la BD
	COMMIT;
    
    --Tratamiento de excepciones.
	    
    EXCEPTION
          WHEN EX_CLIENTE_EXISTE THEN 
            RSP:='ERROR: PRACL001: El cliente con el NIF o CIF '|| codigo ||' ya ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
 
          WHEN EX_CLIENTE_NOMBRE_NULL THEN 
            RSP:='ERROR: PRACL002: El nombre del cliente '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
      
          WHEN EX_CLIENTE_DIRECCION_NULL THEN
             RSP:='ERROR: PRACL003:  La direccin del cliente '||codigo||' no puede tener un valor nulo';
             INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
             COMMIT;	       
	   
          WHEN EX_CLIENTE_LOCALIDAD_NULL THEN 
             RSP:='ERROR: PRACL004:  La localidad del cliente '||codigo||' no puede tener un valor nulo';
             INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
             COMMIT;
                   
          WHEN EX_CLIENTE_PAIS_NULL THEN 
             RSP:='ERROR: PRACL005:  El pas del cliente '||codigo||' no puede tener un valor nulo';
             INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
             COMMIT;		
                    
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	    COMMIT;
      END;