/*Procedimiento que a partir de la lectura del consumo de los contadores  permite establecer la
 *produccin de una Central de produccin en funcin de este. Se ha supuesto un incremento en la
 *produccin de Central de un 25 % sobre el consumo de los contadores. Para determinar el consumo
 *de cada central se ha repartido el consumo de cada contador entre las Centrales de distribucin
 *y el resultado entre la lneras de alimentacin. Adems se actualizan las tablas del 
 *mdulo estadstico y se establece el valor de la energa emitida por la Centrales de distribucin.
 *PARAMETROS IN: id_cont(Variable de tipo NUMBER, contiene el identificador del contador)
 *               fecha_lect (Variable de tipo DATE, contiene la fecha de la lectura del contador)
 *               consumo (Variable de tipo NUMBER, contiene el consumo mensual del contador)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_alta_historico(id_cont IN NUMBER, fecha_lect IN DATE, consumo IN NUMBER, RSP OUT NOCOPY CLOB) AS

        --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;

        --Declaracin de variables
    mes_lectura INTEGER;
    anyo_lectura INTEGER;
    id_central_p VARCHAR2(10);
    cod_central_p_aux VARCHAR2(10);
    cod_linea NUMBER(10);
    num_centrales INTEGER;
    num_lineas INTEGER;
    num_lineas_por INTEGER;
        --Variables para la actualizacin de la produccin y del mdulo estadstico apartado 5
    maxima_produccion NUMBER (10);
    produccion_mes NUMBER(8);
    produccion_acumulada NUMBER (10);
    produccion_acumulada_h NUMBER (10);
    produccion_central NUMBER(10);
    produccion_central_h NUMBER(10);
    num_centrales_p NUMBER(10);
    new_porcentaje NUMBER(5,2);
        --Variable para la actualizacin de la distribucin
    distribucion_energia NUMBER (16,2);
        --Variables mdulo estadstico, apartado 1,3 y 4
    consumo_lectura NUMBER(10,2);
    consumo_centrales NUMBER(10,2);
    consumo_linea NUMBER(10,2);
    consumo_acumulado_linea NUMBER(16,2);
    num_total_lineas INTEGER;
    porcentaje NUMBER(3);
    porcentaje_lineas NUMBER(5,2);
        --Variables mdulo estadistico, apartado 2.
    num_contadores INTEGER;
    consumo_medio NUMBER(10,2);
    consumo_medio_acumulado NUMBER(16,2);

        --Variables mdulo estadstico, apartado 6 y 7.
    cod_contador NUMBER(10);
    consumo_acumulado_contador NUMBER(16,2);
    num_clientes INTEGER;
        --Variables de comprobacin
    existe_consumo INTEGER;
       --Variables para identificar la central de distribucion
    dir VARCHAR2(50);
    loc VARCHAR2(50);
    ps VARCHAR2(25);
        --Declaracion de cursor
    CENTRALES_P SYS_REFCURSOR;
    CURSOR_AUXILIAR SYS_REFCURSOR;
    CURSOR_AUXILIAR_DOS SYS_REFCURSOR;
        
    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_historico';
      
        --Creamos una cadena de texto con los parametros de entrada.

        In_log:= (id_cont||'  ||  ' || fecha_lect ||'  ||  ' || consumo);  
	
	--Obtenemos la produccin absoluta a partir del consumo suponiendo un incremento del  25%.  

	produccion_mes:=ABS(consumo+(consumo*25/100));
	
      	--Dividimos la produccin absoluta y el consumo entre el nmero de centrales de distribucin que suministran al contador

	SELECT COUNT(*) INTO num_centrales FROM tb_Distribucion td WHERE td.contador=id_cont;
        produccion_mes:=produccion_mes/ num_centrales;
	consumo_centrales:=consumo/num_centrales;
	--Obtenemos el mes y el ao de la fecha de la lectura
	SELECT EXTRACT(MONTH FROM fecha_lect)INTO mes_lectura FROM DUAL;
	SELECT EXTRACT(YEAR FROM fecha_lect)INTO anyo_lectura FROM DUAL;
                
       --Abrimos el cursor para recorrer los datos de las centrales y las lneas asociadas al contador.
        
        OPEN CENTRALES_P FOR
            SELECT tp.codigo_central, ta.direccion, ta.localidad, ta.pais, ta.id_linea FROM TB_CENTRALES_P tp INNER JOIN (TB_LINEAS tl INNER JOIN 
            (TB_ALIMENTACION ta INNER JOIN TB_DISTRIBUCION td ON  ta.direccion=td.direccion AND ta.localidad=td.localidad AND ta.pais=td.pais)
            ON codigo_linea=id_linea) ON codigo_central=central_p 
            WHERE contador=id_cont;
        
        --Recorremos el cursor.  
        LOOP FETCH CENTRALES_P INTO id_central_p,dir,loc,ps,cod_linea;
        --Aseguramos la terminacin del bucle cuando no haya ms filas.
        EXIT WHEN CENTRALES_P%NOTFOUND;          
            
            --Determinamos el nmero de lneas usado por cada central de distribucin para repartir el consumo entre ellas.
            SELECT COUNT(*) INTO num_lineas FROM tb_Alimentacion ta WHERE ta.direccion=dir AND ta.localidad=loc AND ta.pais=ps;              
                     
            --Actualizamos la energa emitida por la central de distribucin.
            SELECT energia_emitida INTO distribucion_energia FROM tb_Centrales_D tcd WHERE tcd.direccion=dir AND tcd.localidad=loc AND tcd.pais=ps;
            distribucion_energia:=distribucion_energia+(consumo_centrales/num_lineas);
 
            UPDATE tb_Centrales_D tcd SET energia_emitida=distribucion_energia WHERE tcd.direccion=dir AND tcd.localidad=loc AND tcd.pais=ps;

            --Comprobamos si ya existen datos de produccin en el mes de la lectura
            SELECT COUNT(*)INTO existe_consumo FROM tb_Historico th WHERE th.mes=mes_lectura AND th.anyo=anyo_lectura AND th.central_p=id_central_p;
            
            --Acumulamos al historico la produccin dividiendo est entre las centrales de produccin               
            IF(existe_consumo=0)THEN
                  produccion_acumulada:=produccion_mes/num_lineas;
                  INSERT INTO tb_Historico VALUES (id_central_p, mes_lectura,anyo_lectura, produccion_acumulada);
            ELSE
                  SELECT produccion INTO produccion_acumulada FROM tb_Historico th WHERE th.mes=mes_lectura AND th.anyo=anyo_lectura AND th.central_p=id_central_p;
                  produccion_acumulada:=(produccion_mes/num_lineas) + produccion_acumulada;
                  UPDATE tb_Historico SET produccion=produccion_acumulada WHERE mes=mes_lectura AND anyo=anyo_lectura AND central_p=id_central_p;
            END IF;
                        
            --Comprobamos si la produccin del mes supera a la produccin mxima de la central en un mes
            SELECT maxima_produccion_mes INTO maxima_produccion FROM tb_Centrales_P WHERE codigo_central=id_central_p;
                          
            IF(maxima_produccion < produccion_acumulada)THEN 
                  UPDATE tb_Centrales_P SET maxima_produccion_mes=produccion_acumulada WHERE codigo_central=id_central_p;
            END IF;
                                     
            /*Mdulo estadstico, apartado 1 
	     *Establecemos el consumo de los contadores conectados a una central  de produccin a partir del consumo de la lectur
	     *del mes.		     
             */

	    	--Comprobamos si ya existen datos sobre el consumo de los contadores de la Central de produccin.
            SELECT COUNT(*)INTO existe_consumo FROM me_Central_consumo_contadores WHERE codigo_central_p=id_central_p; 
            
	    	--Si no existen datos simplemente aadimos el consumo relacionado directamente con la central de produccin
		
            IF(existe_consumo=0)THEN
                  consumo_lectura:=consumo_centrales/num_lineas;
                  INSERT INTO me_Central_consumo_contadores VALUES(id_central_p, consumo_lectura);

		--Si existen datos obtenemos el valor del consumo acumulado y le aadimos el nuevo consumo
            ELSE
                  SELECT consumo_acumulado INTO consumo_lectura FROM me_Central_consumo_contadores WHERE codigo_central_p=id_central_p;
                  consumo_lectura:=consumo_lectura+(consumo_centrales/num_lineas);
                  UPDATE me_Central_consumo_contadores SET consumo_acumulado=consumo_lectura WHERE codigo_central_p=id_central_p;  
            END IF;
            /*Fin Mdulo estadstico, apartado 1 */
           
            /* Mdulo estadstico, apartado 2 
             * Se ha de tener en cuenta que el consumo de cada contador se ha dividido entre las distintas centrales
             * de distribucin a las que estn conectados, sin embargo cada una de las lineas conectadas a
             * las centrales de distribucin computa el consumo de la central como si fuera la nica lnea que la alimentara.
             */

		--Obtenemos el nmero total de contadores que son alimentados por la lnea
            SELECT COUNT(DISTINCT td.contador)INTO num_contadores FROM tb_Alimentacion tal 
            INNER JOIN tb_distribucion td ON tal.direccion=td.direccion AND tal.localidad=td.localidad AND tal.pais=td.pais
            WHERE tal.id_linea=cod_linea;
           
		--Comprobamos si ya existen datos para en la tabla estadstica para la lnea que alimenta el contador 
		--de la lectura.
            SELECT COUNT(*)INTO existe_consumo FROM me_Valor_medio_consumo WHERE id_linea=cod_linea AND anyo=anyo_lectura;

		--Calculamos el consumo medio en funcin del nmero de contadores y del valor de la lectura.
            consumo_medio:=consumo/num_contadores;
            	
		--Si no existe consumo aadimos el consumo medio calculado a la tabla estadstica          
            IF(existe_consumo=0) THEN
                  INSERT INTO me_Valor_medio_consumo VALUES(cod_linea, anyo_lectura, consumo_medio);

		--Si existen datos obtenemos el consumo medio acumulado y le aadimos el nuevo consumo medio. 
            ELSE
                  SElECT valor_medio_consumo INTO consumo_medio_acumulado FROM me_Valor_medio_consumo WHERE id_linea=cod_linea AND anyo=anyo_lectura;
                  consumo_medio_acumulado:=consumo_medio_acumulado+consumo_medio;
                  UPDATE me_Valor_medio_consumo SET valor_medio_consumo=consumo_medio_acumulado WHERE id_linea=cod_linea AND anyo=anyo_lectura;
            END IF;
            /*Fin mdulo estadstico, apartado 2 */
         
            /*Mdulo estadstico, apartado 3 
             *Como en el apartado anterior y siguiendo las indicaciones del cliente,  se ha computado el consumo 
             *de cada lnea como si fuera la nica que alimenta a la central de distribucin.
             */

		--Comprobamos si ya existen datos introducidos en la tabla
             SELECT COUNT(*)INTO existe_consumo FROM me_Linea_mas_cargada;

		--Si no existen aadimos el consumo en funcin de la energia suministrada a las
		--Centrales de distribucin por la lnea actual.
             IF(existe_consumo=0)THEN
                  INSERT INTO me_Linea_mas_cargada VALUES(1,cod_linea,consumo_centrales);
		--Obtenemos la mxima energa consumida por una lnea y lo comparamos con el 
		--valor del total de la energa consumida por la lnea actual, si energa consumida
		--por la lnea actual es mayor actualizamos la tabla estadstica
             ELSE
                  SELECT max_total_consumo INTO consumo_lectura FROM me_Linea_mas_cargada;
                  SELECT SUM(NVL(energia_emitida/num_lineas,0))INTO consumo_acumulado_linea FROM tb_Centrales_D tcd
                  INNER JOIN tb_Alimentacion tal ON tal.direccion=tcd.direccion AND tal.localidad=tcd.localidad AND tal.pais=tcd.pais
                  WHERE tal.id_linea=cod_linea;

                  IF(consumo_lectura < consumo_acumulado_linea) THEN
                      UPDATE me_Linea_mas_cargada SET id_linea=cod_linea, max_total_consumo=consumo_acumulado_linea WHERE identificador=1;
                  END IF;
             END IF;
             /*Fin mdulo estadstico, apartado 3 */
            
             /*Mdulo estadstico, apartado 4 
             /*El consumo de las lneas esta calculado suponiendo que estas computan el consumo de cada una de las 
              *centrales de distribucin a las que alimentan como si fuera la nica linea.
              */  

		--Comprobamos si ya existen valores en la tabla estadstica           
             SELECT COUNT(*)INTO existe_consumo FROM me_porcentaje_lineas_mas_50 WHERE anyo=anyo_lectura;

		--Obtenemos el 50% (o lo que es lo mismo la mitad) del consumo de todos los contadores en el ao de la lectura.
             SELECT SUM(consumo_mes/2)INTO consumo_lectura FROM tb_Lecturas tle WHERE anyo_lectura =(SELECT EXTRACT(YEAR FROM tle.fecha)FROM DUAL);

		--Si es la primera lectura en el ao calculamos su 50%
             IF(consumo_lectura IS NULL)THEN consumo_lectura:=consumo/2;

		--Si no aadimos el 50% de la lectura actual al 50% del total de las lecturas del ao.
             ELSE consumo_lectura:=consumo_lectura+(consumo/2);
             END IF;

		--Si no existen datos para el ao de la lectura en la tabla estadstica insertamos el actual pero en porcentaje.
             IF (existe_consumo=0)THEN
                          INSERT INTO me_porcentaje_lineas_mas_50 VALUES(anyo_lectura,100/num_total_lineas);
             ELSE
                          SELECT COUNT(*)INTO num_total_lineas FROM tb_Lineas;
                                 SELECT SUM(consumo_mes)INTO consumo_acumulado_linea 
                                 FROM tb_Lecturas tle INNER JOIN(tb_Distribucion tdi INNER JOIN tb_Alimentacion tal
                                 ON tdi.direccion=tal.direccion AND tdi.localidad=tal.localidad AND tdi.pais=tal.pais) ON tle.contador=tdi.contador
                          WHERE tal.id_linea=cod_linea AND anyo_lectura=(SELECT EXTRACT(YEAR FROM tle.fecha)FROM DUAL);
          
                          IF(consumo_acumulado_linea IS NULL)THEN 
                                    porcentaje:=0;
                                    consumo_acumulado_linea:=(consumo/num_centrales);
                          ELSE 
                                    IF(consumo_acumulado_linea > consumo_lectura)THEN porcentaje:=1;
                                    ELSE porcentaje:=0;
                                    END IF;
                                    consumo_acumulado_linea:=((consumo_acumulado_linea+consumo)/num_centrales);
                          END IF;
                          --Comprobamos que la lnea supera el 50% del consumo y que no estaba ya incluida en el porcentaje
                          --de ser asi, la aadimos
                          IF (consumo_acumulado_linea > consumo_lectura AND porcentaje=0) THEN
                                      SELECT ROUND(
                                      (SELECT PORCENTAJE_MAS_50 FROM me_porcentaje_lineas_mas_50 WHERE anyo=anyo_lectura)*num_total_lineas/100,0)
                                      INTO num_lineas_por FROM DUAL;
                                      num_lineas_por:=num_lineas_por+1;
                          END IF;
                          --Comprobamos si la lnea supera el 50% del consumo, si no lo superara y estuviera incluida
                          --en el porcentaje anterior la eliminamos.
                          IF (consumo_acumulado_linea < consumo_lectura AND porcentaje=1) THEN
                                      SELECT ROUND(
                                      (SELECT PORCENTAJE_MAS_50 FROM me_porcentaje_lineas_mas_50 WHERE anyo=anyo_lectura)*num_total_lineas/100,0)
                                      INTO num_lineas_por FROM DUAL;
                                      num_lineas_por:=num_lineas_por-1;
                          END IF;
                          
                          --Recalculamos el porcentaje de lneas.
                          IF(num_lineas_por IS NULL) THEN 
                                porcentaje_lineas:=0;
                          ELSE
                                porcentaje_lineas:=num_lineas_por*100/num_total_lineas;
                         END IF;
                         UPDATE me_porcentaje_lineas_mas_50 SET porcentaje_mas_50=porcentaje_lineas WHERE anyo=anyo_lectura;
             END IF;
             /*Fin mdulo estadstico 4 */
             
        END LOOP;
        CLOSE CENTRALES_P;
        
        /*Mdulo estadstico 5 
        *Calculamos el nmero de centrales de produccin que producen menos del 30% de la produccin
        *total de un ao dado
        */
        
        --Limpiamos la tabla
        DELETE FROM me_numero_centrales_menos_30 WHERE anyo=anyo_lectura;
        num_centrales_p:=0;
        
        --Recorremos la tabla de los historicos de produccin para calcular lo solicitado.
        OPEN CURSOR_AUXILIAR_DOS FOR
             SELECT central_p  FROM tb_Historico WHERE anyo=anyo_lectura
             GROUP BY central_p;
             --Recorremos el cursor.  
             LOOP FETCH CURSOR_AUXILIAR_DOS INTO cod_central_p_aux;
             --Aseguramos la terminacin del bucle cuando no haya ms filas.
             EXIT WHEN CURSOR_AUXILIAR_DOS%NOTFOUND;  
			
			--Obtenemos el total de la produccin del ao de la lectura
                    SELECT SUM(produccion)INTO produccion_acumulada_h FROM tb_Historico WHERE anyo=anyo_lectura;

			--Obtenemos el total de la produccin de la central de produccin en el ao de la lectura
                    SELECT SUM(produccion)INTO produccion_central_h FROM tb_Historico WHERE anyo=anyo_lectura AND central_p=cod_central_p_aux;

			--Obtenemos el porcentaje de produccin y comprobamos si es menor del 30%
                    new_porcentaje:=produccion_central_h*100/produccion_acumulada_h;
                    IF(new_porcentaje <30)THEN
                          num_centrales_p:=num_centrales_p+1;
                    ELSE 
                          num_centrales_p:=num_centrales_p+0;
                    END IF;

			--Comprobamos si ya existen datos introducidos en la tabla estadstica para el ao de la lectura
                    SELECT COUNT(*)INTO existe_consumo FROM me_numero_centrales_menos_30 WHERE anyo=anyo_lectura;

			--Si no existen aadimos los resultados obtenidos 
                    IF(existe_consumo=0)THEN
                          INSERT INTO me_numero_centrales_menos_30 VALUES (anyo_lectura, num_centrales_p);

			--Si existen actualizamos a partir de los resultados obtenidos.
                    ELSE
                          UPDATE me_numero_centrales_menos_30 SET num_centrales=num_centrales_p WHERE anyo=anyo_lectura;
                    END IF;
             END LOOP;
        CLOSE CURSOR_AUXILIAR_DOS;
        /*Fin mdulo estadstico 5*/  

        /*Mdulo estadstico, apartado 6
	 *Top de los 10 contadores que han tenido ms consumo
	 *historicamente
	 */

	--Obtenemos el valor del consumo acumulado del contador que ocupara la dcima posicin en el TOP de contadores
        SELECT MIN(consumo_acumulado)INTO consumo_lectura FROM
                (SELECT consumo_acumulado FROM tb_Contadores ORDER BY consumo_acumulado DESC)
        WHERE ROWNUM <=10;
      
	--Obtenemos el valor del consumo acumulado del nuevo contador y le aadimos el consumo actual
        SELECT consumo_acumulado INTO consumo_acumulado_contador FROM tb_contadores WHERE id_contador=id_cont;
        consumo_acumulado_contador:=consumo_acumulado_contador+consumo;
        
        --Comprobamos si el contador ya esta incorporado en el TOP
	SELECT COUNT(*)INTO existe_consumo FROM me_top_contadores_mas_consumo WHERE id_contador=id_cont;

	--Obtenemos el nmero de contadores incorporados al TOP
        SELECT COUNT(*)INTO num_contadores FROM me_top_contadores_mas_consumo;

	--Si el consumo de contador es mayor que el consumo del contador con mnimo consumo en el TOP
	-- lo introducimos en la tabla estadstica dependiendo de si ya existe en la tabla y de si la 
	--tabla ya tiene 10 contadores
        IF((existe_consumo=0) AND (consumo_acumulado_contador > consumo_lectura)) THEN
                IF(num_contadores=10)  THEN
                      DELETE FROM me_top_contadores_mas_consumo 
                      WHERE id_contador =(SELECT id_contador FROM tb_contadores WHERE consumo_acumulado=consumo_lectura);
                END IF;
                INSERT INTO me_top_contadores_mas_consumo VALUES(id_cont,consumo_acumulado_contador);
        ELSE
                UPDATE me_top_contadores_mas_consumo SET consumo=consumo_acumulado_contador WHERE id_contador=id_cont; 
        END IF;  
	
	--Utilizamos un cursor una vez introducido el nuevo contador
        --para ordenar la tabla por el consumo  de los contadores en orden descendente
        OPEN CURSOR_AUXILIAR FOR
              SELECT id_contador, consumo FROM me_top_contadores_mas_consumo ORDER BY consumo DESC;
              --Recorremos el cursor y ordenamos la tabla.  
              LOOP FETCH CURSOR_AUXILIAR INTO cod_contador, consumo_acumulado_contador;
              --Aseguramos la terminacin del bucle cuando no haya ms filas.
              EXIT WHEN CURSOR_AUXILIAR%NOTFOUND;  
                      DELETE FROM me_top_contadores_mas_consumo WHERE id_contador=cod_contador;
                      INSERT INTO me_top_contadores_mas_consumo VALUES(cod_contador,consumo_acumulado_contador);
              END LOOP;
        CLOSE CURSOR_AUXILIAR;
        /*Fin mdulo estadstico, apartado 6*/

          
        /*Modulo estadstico, apartado 7
	 *Clculo del consumo medio de todos los clientes
	*/

	--Obtenemos el nmero total de clientes	
        SELECT COUNT(*)INTO num_clientes FROM tb_Clientes;

	--Comprobamos si ya existen datos en la tabla estadstica
        SELECT COUNT(*)INTO existe_consumo FROM me_consumo_medio_clientes;

	--Si no existen aadimos el consumo actual partido entre el nmero total de clientes.
        IF(existe_consumo=0) THEN
               INSERT INTO me_consumo_medio_clientes VALUES(consumo/num_clientes); 

	--Si existen obtenemos el valor del consumo acumulado, le aadimos el nuevo consumo y los dividimos entre todos
	--clientes para obtener el consumo medio de estos.
        ELSE
               SELECT SUM(CONSUMO_ACUMULADO) INTO consumo_acumulado_contador FROM tb_Contadores;
               UPDATE me_consumo_medio_clientes mcmc SET mcmc.consumo_medio=(consumo_acumulado_contador+consumo)/num_clientes;
        END IF;
       /*Fin mdulo estadstico, apartado 7*/
      
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP); 

     --Los posibles errores fueron capturados el introducir la lectura por lo que capturamos aquellos que no hemos previsto.   
     EXCEPTION
     
         WHEN OTHERS THEN 
         RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
         INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;




