--Alta Lnea--
/*Procedimiento que a partir de los parmetros de entrada aade una nueva lnea a la base de datos
 *PARAMETROS IN: codigo (Variable de tipo NUMBER, contiene la identificacion de la lnea)
 *		 capacidad_linea (Variable de tipo NUMBER, contiene la capacidad mximad de energa que la lnea puede transportar)
 *		 codigo_central_p (Variable de tipo Varchar2, contiene la identificacin nica de la central de produccin a la que la lnea est conectada)
 *		 RSP (RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno). 	    
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
create or replace PROCEDURE pr_alta_linea(codigo IN NUMBER, capacidad_linea IN NUMBER, codigo_central_p IN VARCHAR2 , RSP OUT NOCOPY CLOB) AS
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;
    
    --variables de comprobacion
    existe_linea INTEGER;
    existe_central INTEGER;

    --Declaracin de excepciones
    EX_LINEA_EXISTE EXCEPTION;
    EX_LINEA_CAP_NULL EXCEPTION;
    EX_LINEA_CENTRAL_NULL EXCEPTION;
    EX_CENTRAL_NO_EXISTE EXCEPTION;
     
    --Ejecucin del procedimiento.
    BEGIN
          
        --Obtenemos la fecha del ejecucin de procedimiento de los datos del sistema
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_linea';
      
        --Creamos una cadena de texto con los parametros de entrada separados por barras.
        In_log:= (codigo||'  ||  ' || capacidad_linea ||'  ||  ' || codigo_central_p); 
       
        --Comprobamos que no existe una lnea con la misma identificacin lo cual
        --supondra una violacin de la PK.
        SELECT COUNT(*)INTO existe_linea FROM tb_Lineas lin WHERE lin.codigo_linea=codigo;
        
        --Si existe una violacin de la PK capturamos la excepcin para su posterior tratamiento.
        IF(existe_linea > 0) THEN RAISE EX_LINEA_EXISTE; END IF;
        
        --Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(capacidad_linea is NULL)THEN RAISE EX_LINEA_CAP_NULL; END IF;
        IF(codigo_central_p is NULL)THEN RAISE EX_lINEA_CENTRAL_NULL; END IF;
        
        --Comprobamos que la central de produccin existe.
        SELECT COUNT(*)INTO existe_central FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo_central_p;
        IF(existe_central=0)THEN RAISE EX_CENTRAL_NO_EXISTE;END IF;

        --Si no se han producido excepciones realizamos la inserccin.    
        INSERT INTO tb_Lineas VALUES(codigo,capacidad_linea,codigo_central_p);
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos commit para confirmar la inserccin definitiva en la BD
	COMMIT;        

    --Tratamiento de excepciones
    EXCEPTION
          WHEN EX_LINEA_EXISTE THEN 
            RSP:='ERROR: PRALI001: La lnea con el cdigo '|| codigo ||' ya ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT; 

          WHEN EX_LINEA_CAP_NULL THEN 
            RSP:='ERROR: PRALI002: La capacidad de la lnea n '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;             
      
          WHEN EX_LINEA_CENTRAL_NULL THEN
             RSP:='ERROR: PRALI003: La central de produccin asociada a la lnea n '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;       
            
          WHEN EX_CENTRAL_NO_EXISTE THEN 
            RSP:='ERROR: PRALI004: La central de produccin con codigo: ' ||codigo_central_p||', no ha sido dado de alta en la base de datos';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;             
        
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;       
END;
