--Baja alimentacin-
/*Procedimiento que a partir de los parmetros de entrada da borra una asociacin entre las lneas y las centrales de distribucin
 *PARAMETROS IN: dir (variable de tipo Varchar2, contiene la direccin(calle, avenida, polgono, etc). de la central)
 *		 loc (variable de tipo varchar2, contiene la localidad (Barcelona, Madrid, etc) de la central)
 *		 ps  (Variable de tipo varchar2, contiene el pas donde est situada la central)
 *		 linea_alimentacion (variable de tipo NUMBER, contiene el identificador nico de la lnea)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */

CREATE OR REPLACE
PROCEDURE pr_baja_alimentacion(dir IN VARCHAR2, loc IN VARCHAR2, ps IN VARCHAR2 , linea_alimentacion NUMBER, RSP OUT NOCOPY CLOB) AS
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(25);
    In_log CLOB;

    --Variables de comprobacin
    existe_alimentacion INTEGER;
    
    --Declaracin de excepciones
    EX_ALIMENTACION_NO_EXISTE EXCEPTION;
    
    --Ejecucin del procedimiento.
    BEGIN
          
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_baja_alimentacion';
      
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (dir||'  ||  ' || loc ||'  ||  ' || ps ||'  ||  ' ||linea_alimentacion);
	
	--Comprobamos que la asociacin entre la central y la lnea existe.
	SELECT COUNT(*)INTO existe_alimentacion FROM tb_Alimentacion ta WHERE ta.direccion=dir AND ta.localidad=loc AND ta.pais=ps AND ta.id_linea=linea_alimentacion;
    	IF(existe_alimentacion = 0)THEN RAISE EX_ALIMENTACION_NO_EXISTE;END IF;
     
        --Si no se han producido excepciones damos de baja la asociacin.    
        DELETE FROM tb_Alimentacion ta WHERE ta.direccion=dir AND ta.localidad=loc AND ta.pais=ps AND ta.id_linea=linea_alimentacion;
        RSP:='OK';
	
	--Actualizamos la tabla de logs.
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos Commit para realizar la insercin de los datos en la base de datos.
        COMMIT;
      
    --Tratamiento de excepciones
    EXCEPTION
          
          WHEN EX_ALIMENTACION_NO_EXISTE THEN 
            RSP:='ERROR: PRBAL001: La asociacin entre la lnea y la central de distribucin NO existe';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
 
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;

    END;