--Alta contrato--
/*Procedimiento que a partir del parmetro de entrada da de baja un contrato a la base de datos
 *PARAM IN: contrato (variable de tipo Varchar2, contiene el identificador nico del contrato)
 *	    RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_baja_contrato(contrato IN VARCHAR2, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;
    
    --Variable de comprobacion
    existe_contrato INT;
    contrato_activo DATE;

    --Declaracin de excepciones
    EX_CONTRATO_NO_EXISTE EXCEPTION;
    EX_FECHA_BAJA_NULL EXCEPTION;
   
    --Ejecucin del procedimiento.
    BEGIN
        
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_baja_contrato';
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (contrato); 
       
        --Comprobamos que existe el contrato.
        SELECT COUNT(*)INTO existe_contrato FROM tb_Contratos tco WHERE tco.id_contrato=contrato;
        IF(existe_contrato = 0) THEN RAISE EX_CONTRATO_NO_EXISTE; END IF;
       
	--Comprobamos que el contrato tiene fecha de baja
	SELECT fecha_baja INTO contrato_activo FROM tb_Contratos tco WHERE tco.id_contrato=contrato;
        IF(contrato_activo IS NULL)THEN RAISE EX_FECHA_BAJA_NULL; END IF;
 
        --No se existen excepciones por lo que damos la baja del contrato
	DELETE FROM tb_Contratos tco WHERE tco.id_contrato=contrato;
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	
	--Ejecutamos commit para confirmar la inserccin definitiva en la BD.
	COMMIT;   

    --Tratamiento de excepciones
   
    EXCEPTION
          WHEN EX_CONTRATO_NO_EXISTE THEN 
            RSP:='ERROR: PRACN001: El contrato '|| contrato ||' no existe';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
          
          WHEN EX_FECHA_BAJA_NULL THEN
             RSP:='ERROR: PRACN002: El contrato '||contrato||' no ha sido dado de baja';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
            
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;

      END;