--Baja distribucion--
/*Procedimiento que a partir de los parmetros de entrada da de baja una asociacin en la tabla distribucin 
 *PARAMETROS IN: id_cont (variable de tipo NUMBER, contiene el identificador nico del contador)
 *		 dir (variable de tipo Varchar2, contiene la direccin -calle, avenida, polgono, etc- de la central)
 *		 loc (variable de tipo varchar2, contiene la localidad (Barcelona, Madrid, etc) de la central)
 *		 ps  (Variable de tipo varchar2, contiene el pas donde est situada la central)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
		 
CREATE OR REPLACE
PROCEDURE pr_baja_distribucion(id_cont IN NUMBER, dir IN VARCHAR2, loc IN VARCHAR2, ps IN VARCHAR2 , RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(25);
    In_log CLOB;
    
    --Variables de comprobacin
    existe_distribucion INTEGER;
    
    --Declaracin de excepciones
    EX_DISTRIBUCION_NO_EXISTE EXCEPTION;
    
    --Ejecucin del procedimiento.
    BEGIN  
    
        --Obtenemos la fecha de ejecucin del proceso a partir de los datos del sistema.
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_baja_distribucion';
      
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (id_cont||'  ||  '||dir||'  ||  ' || loc ||'  ||  ' || ps);

	--Comprobamos que la distribucin existe.
	SELECT COUNT(*)INTO existe_distribucion FROM tb_Distribucion td WHERE td.contador=id_cont AND td.direccion=dir AND td.localidad=loc AND td.pais=ps;
    	IF(existe_distribucion = 0)THEN RAISE EX_DISTRIBUCION_NO_EXISTE;END IF;
     
        --Si no se han producido excepciones damos de baja la asociacin.    
        DELETE FROM tb_Distribucion td WHERE td.contador=id_cont AND td.direccion=dir AND td.localidad=loc AND td.pais=ps;
        RSP:='OK';
	
	--Actualizamos la tabla de logs.
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos Commit para realizar la insercin definitiva de los datos en la base de datos.
        COMMIT;
      
    --Tratamiento de excepciones
    EXCEPTION

	  WHEN EX_DISTRIBUCION_NO_EXISTE THEN
	    RSP:='ERROR: PRBDI001: El contador y la central de distribucin ya no estn asociados en la BD';
	    INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
            
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;

    END;
