--Modificacin Central Distribucin--
/*Procedimiento que a partir de los parmetros de modifica el valor del suministro mximo de  una central de distribucin
 *identificada por su clave primaria.
 *PARAMETROS IN: dir (variable de tipo Varchar2, contiene la direccin(calle, avenida, polgono, etc). de la central)
 *		 loc (variable de tipo varchar2, contiene la localidad (Barcelona, Madrid, etc) de la central)
 *		 ps  (Variable de tipo varchar2, contiene el pas donde est situada la central)
 *		 sumi_max (Variable de tipo NUMBER, contiene el mximo de energa permitada por la central)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */

CREATE OR REPLACE PROCEDURE pr_modifica_central_d(dir IN VARCHAR2, loc IN VARCHAR2, ps IN VARCHAR2 , sumi_max IN NUMBER, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables 
    
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;
    existe_central INTEGER;
    
    --Declaracin de excepciones
    EX_CENTRAL_NO_EXISTE EXCEPTION;
    EX_CENTRAL_DIR_LOC_PS_NULL EXCEPTION;
    EX_CENTRAL_SUMINISTRO_NULL EXCEPTION;
     
    --Ejecucin del procedimiento.
    BEGIN

	--Obtenemos la fecha de ejecucin del procedimiento a partir de los datos del sistema.
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_central_d';
      
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (dir||'  ||  ' || loc ||'  ||  ' || ps||'  ||  ' || sumi_max );
        
        --Comprobamos que los valores que forman parte de la PK de la central no son nulos.
        IF(dir IS NULL OR loc IS NULL OR ps IS NULL)THEN RAISE EX_CENTRAL_DIR_LOC_PS_NULL;END IF;
        
        --Comprobamos que la central ha sido dada de alta en la BD.
	SELECT COUNT(*)INTO existe_central FROM tb_Centrales_D cd WHERE(cd.direccion=dir AND cd.localidad=loc AND cd.pais=ps);
        IF(existe_central =0)THEN RAISE EX_CENTRAL_NO_EXISTE;END IF;
        
        --Comprobamos que la variable a modificar no tenga un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(sumi_max is NULL)THEN RAISE EX_CENTRAL_SUMINISTRO_NULL; END IF;
       
        --Si no se han producido excepciones modificamos la central.    
        UPDATE tb_Centrales_D SET suministro_maximo=sumi_max WHERE(direccion=dir AND localidad=loc AND pais=ps);
        RSP:='OK';
	
	--Actualizamos la tabla de logs.
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos Commit para realizar la insercin definitiva en la base de datos.
        COMMIT;
      
    --Tratamiento de excepciones
    EXCEPTION
          WHEN EX_CENTRAL_NO_EXISTE THEN 
            RSP:='ERROR: PRMCD001: La central de distribucin no ha sido dada de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
            
          WHEN EX_CENTRAL_DIR_LOC_PS_NULL THEN
             RSP:='ERROR: PRMCD002: La clave primaria de la central no puede tener valores nulos';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
                  
          WHEN EX_CENTRAL_SUMINISTRO_NULL THEN 
            RSP:='ERROR: PRMCD003: El valor del suministro mximo de la central no puede ser nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
            
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
    END;

