--Modifica Central Produccin--
/*Procedimiento que a partir de los parmetros de entrada se modifica una central de Produccin a la base de datos
 *los atributos tipo, direccion, localidad y pais no son modificables. El atributo de mxima produccin se modificar
 *al insertar o modificar las lecturas.
 *PARAMETROS IN: codigo (variable tipo varchar2, contendr el identificador nico de la central)
 *		 consumo_func (Variable de tipo NUMBER, contiene la energa que la central necesita consumir para funcionar)
 *		 residuos_rad (variable de tipo NUMBER, contiene el volumen de residuos radiactivos generados por las nucleares)
 *		 emisiones_co2 (Variable de tipo NUMBER, contiene los KG de emisin de CO2 generados por las centrales Termica y Carbn)
 *	 	 molinos (Variable de tipo NUMBER, contiene el nmero de molinos instalados en una central Elica.
 *		 paneles (Variable de tipo NUMBER, contiene el nmero de paneles que necesita una central Solar para funcionar)
 *		 ultima_inspeccion (Variable de tipo DATE, contiene la fecha de la ltima inspeccin de la Central). 
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno)
 *RETURN: Parmetro RSP, variable de tipo CLOB que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */

CREATE OR REPLACE
PROCEDURE pr_modifica_central_p(codigo IN VARCHAR2, consumo_func IN NUMBER, residuos_rad IN NUMBER,emisiones_co2 IN NUMBER, 
molinos IN NUMBER, paneles IN NUMBER, ultima_inspeccion IN DATE, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;

    -- Variable de comprobacion	
    existe INTEGER;
    
    tipo VARCHAR2(10);
    con_fun NUMBER:=consumo_func;
    res_rad NUMBER:=residuos_rad;
    inspeccion DATE:=ultima_inspeccion;
    --Declaracin de excepciones
    EX_CENTRAL_NO_EXISTE EXCEPTION;
    EX_VALORES_NULOS EXCEPTION;
    EX_VALORES_NULOS_TIPO EXCEPTION;
      
    --Ejecucin del procedimiento.
    BEGIN
         
        --obtenemos la fecha de ejecucin del procedimiento a partir de los datos del sistema
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_central_p';
      
        --Creamos una cadena de texto con los parmetros de entrada separados por barras.
        In_log:= (codigo||'  ||  ' || consumo_func ||'  ||  ' || residuos_rad||'  ||  ' || emisiones_co2 ||'  ||  ' || molinos||'  ||  ' || paneles||'  ||  ' || ultima_inspeccion);        
        
	--Comprobamos que la central de produccin existe
        SELECT COUNT(*)INTO existe FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo;
        
        IF(existe= 0) THEN RAISE EX_CENTRAL_NO_EXISTE; END IF;
        
        --Comprobamos el resto de variables
        IF((consumo_func IS NULL) AND (residuos_rad IS NULL) AND (emisiones_co2 IS NULL) AND (molinos IS NULL) AND (paneles IS NULL) AND (ultima_inspeccion IS NULL)) THEN
              RAISE EX_VALORES_NULOS;
        END IF;

	SELECT tipo_central INTO tipo FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo; 

	--Comprobamos que los atributos pasados por parmetro se correspoden con los del tipo de central
	IF(tipo='NUCLEAR' AND (consumo_func is NULL OR residuos_rad IS NULL)) THEN
		IF(emisiones_co2 IS NOT NULL OR molinos IS NOT NULL OR paneles IS NOT NULL)THEN RAISE EX_VALORES_NULOS_TIPO; END IF;
        END IF;
        
        IF((tipo='TERMICA' OR tipo='CARBO') AND (emisiones_co2 IS NULL)) THEN
		 IF(consumo_func IS NOT NULL OR residuos_rad IS NOT NULL OR molinos IS NOT NULL OR paneles IS NOT NULL) THEN RAISE EX_VALORES_NULOS_TIPO; END IF;
        END IF;
        
        IF(tipo='EOLICA' AND molinos IS NULL)THEN
		IF (emisiones_co2 IS NOT NULL OR consumo_func IS NOT NULL OR residuos_rad IS NOT NULL OR paneles IS NOT NULL)THEN RAISE EX_VALORES_NULOS_TIPO;END IF; 
        END IF;
        
        IF(tipo='SOLAR' AND paneles IS NULL) THEN
		IF (emisiones_co2 IS NOT NULL OR consumo_func IS NOT NULL OR residuos_rad IS NOT NULL OR molinos IS NOT NULL) THEN RAISE EX_VALORES_NULOS_TIPO; END IF;
        END IF;
  
        IF(tipo='NUCLEAR' AND consumo_func IS NULL) THEN 
              SELECT energia_funcionamiento INTO con_fun FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo;
        END IF;
        
        IF(tipo='NUCLEAR' AND residuos_rad IS NULL) THEN
              SELECT kg_residuos_radiactivos INTO res_rad FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo;
        END IF;
        
        IF(ultima_inspeccion IS NULL)THEN SELECT fecha_ultima_inspeccion INTO inspeccion FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo; END IF;
        
       --Al no producirse ningn error realizamos la modificacin
	UPDATE tb_Centrales_P SET energia_funcionamiento=con_fun, kg_residuos_radiactivos=res_rad, kg_CO2=emisiones_co2, numero_molinos=molinos, numero_paneles=paneles, fecha_ultima_inspeccion=inspeccion        WHERE codigo_central=codigo;
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
        
	--Ejecutamos Commit para confirmar la insercin definitiva en la base de datos.
        COMMIT;
        
    EXCEPTION
          WHEN EX_CENTRAL_NO_EXISTE THEN 
            RSP:='ERROR: PRMCP001: La central de produccin con el cdigo '|| codigo ||' NO ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
                  
          WHEN EX_VALORES_NULOS THEN
             RSP:='ERROR: PRMCP002: No se han pasado valores para modificar';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
                  
          WHEN EX_VALORES_NULOS_TIPO THEN 
            RSP:='ERROR: PRMCP003: Se han pasado valores para modificar que no se corresponden con el tipo de Central';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
                   
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
      END;

