--Modificacin Cliente--
/*Procedimiento que a partir de los parmetros de entrada modifica los datos de un cliente  identificado por su CIF, NIF, etc. en
 *la base de datos.
 *PARAM IN: identificacion(Variable de tipo Varchar2, contiene el NIF, CIF o cualquier otro identificador nico para el cliente).
 *	    nombre (Variable de tipo Varchar2, contiene el nombre y los apellidos del cliente o la Razn social en caso de empresas)
 *	    direccin (Variable de tipo Varchar2, contiene la direccin completa del cliente -calle, nmero, piso-)
 *	    localidad (Variable de tipo Varchar2, contiene el nombre de la localidad correspondiente al domicilio del cliente)
 *	    pas (Variable de tipo Varchar2, contiene el pas del domiciolio del cliente)
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_modifica_cliente(codigo IN VARCHAR2, nom IN VARCHAR2, dir IN VARCHAR2,loc IN VARCHAR2, ps IN VARCHAR2,  RSP OUT NOCOPY CLOB) AS
   
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;
   
   nombre_cli VARCHAR2(50):=nom;
   dir_cli VARCHAR2(50):=dir;
   loc_cli VARCHAR2(50):=loc;
   ps_cli VARCHAR2(25):=ps;

   --Variable de comprobacin
    existe INTEGER;
    
    --Declaracin de excepciones
    EX_CLIENTE_NO_EXISTE EXCEPTION;
    EX_VALORES_NULOS EXCEPTION;

    --Ejecucin del procedimiento.
    BEGIN
          
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_cliente';
      
        --Creamos una cadena de texto con los parametros de entrada separados por barras.
        In_log:= (codigo||'  ||  ' || nom||'  ||  ' || dir||'  ||  ' || loc||'  ||  ' || ps);   
     
        --Comprobamos que existe el cliente.
        SELECT COUNT(*)INTO existe FROM tb_Clientes c WHERE c.id_cliente=codigo;
        
        --Si no existe en la BD, capturamos la excepcin para su posterior tratamiento.
        IF(existe=0) THEN RAISE EX_CLIENTE_NO_EXISTE; END IF;

        --Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF ((nom is NULL) AND (dir is NULL) AND (loc is NULL) AND (ps is NULL)) 
                THEN RAISE EX_VALORES_NULOS;
        ELSE
		IF(nom is NULL)THEN SELECT nombre INTO nombre_cli FROM tb_Clientes WHERE id_cliente=codigo;END IF;
                IF(dir is NULL)THEN SELECT direccion INTO dir_cli FROM tb_Clientes WHERE id_cliente=codigo;END IF;
                IF(loc is NULL)THEN SELECT localidad INTO loc_cli FROM tb_Clientes WHERE id_cliente=codigo;END IF;
                IF(ps is NULL)THEN SELECT pais INTO ps_cli FROM tb_Clientes WHERE id_cliente=codigo;END IF;
        END IF;

        --Modificamos los datos del cliente
        UPDATE tb_Clientes SET nombre=nombre_cli, direccion=dir_cli, localidad=loc_cli, pais=ps_cli WHERE id_cliente=codigo;
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos commit para confirmar la modificacion en la BD
	COMMIT;
    
    --Tratamiento de excepciones.
	    
    EXCEPTION
          WHEN EX_CLIENTE_NO_EXISTE THEN 
            RSP:='ERROR: PRMCL001: El cliente con el NIF o CIF '|| codigo ||' NO ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
 
          WHEN EX_VALORES_NULOS THEN 
            RSP:='ERROR: PRMCL002: No se ha pasado ningn dato para modificar';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
            COMMIT;
                      
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	    COMMIT;
      END;