--Modificacin contador--
/*Procedimiento que a partir de los parmetros de modifica la direccion de un contador en la base de datos.
 *Las variables modelo, any_fabricacion y fabricante se supone que no han de ser actualizadas, y la variable
 *consumo_acumulado se actualiza al insertar o modificar una lectura en el contador.
 *PARAM IN: id_conta (variable de tipo NUMBER, contiene el identificador del contador)
 *	    direccion (variable de tipo Varchar2, contiene la direccin en la que est ubicado el contador)
 *	    localidad (variable de tipo Varchar2, contiene la localidad en la que se encuentra instalado el contador)
 *	    pais (variable de tipo Varchar2, contiene el pais de ubicacin del contador)
 * 	    RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
  RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_modifica_contador(id_cont IN NUMBER, direccion IN VARCHAR2, localidad IN VARCHAR2, pais IN VARCHAR2, RSP IN OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;
    
    dir_cont VARCHAR2(50):=direccion;
    loc_cont VARCHAR2(50):=localidad;
    ps_cont VARCHAR2(25):=pais;

    --Variable de comprobacin
    existe INTEGER;
    
    --Declaracin de excepciones
    EX_VALORES_NULOS EXCEPTION;
    EX_CONTADOR_NO_EXISTE EXCEPTION;
    
    --Ejecucin del procedimiento.
    BEGIN
          
	--Obtenemos la fecha de ejecucin del procedimiento a partir de los datos del sistema.
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_contador';
      
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (id_cont||'  ||  ' || direccion||'  ||  ' || localidad||'  ||  ' || pais);        
        
	--Comprobamos que el contador existe en la BD.
        SELECT COUNT(*)INTO existe FROM tb_Contadores c WHERE c.id_contador=id_cont;
        
        IF(existe = 0) THEN RAISE EX_CONTADOR_NO_EXISTE; END IF;
        --Comprobamos que el resto de las variables 
	IF((direccion is NULL) AND (localidad is NULL) AND (pais is NULL))THEN
		RAISE EX_VALORES_NULOS;END IF;

        IF(direccion is NULL)THEN
		SELECT direccion INTO dir_cont FROM tb_Contadores WHERE id_contador=id_cont; 
	END IF;
        
	IF(localidad is NULL)THEN 
		SELECT localidad INTO loc_cont FROM tb_Contadores WHERE id_contador=id_cont;
	END IF;
        
	IF(pais is NULL)THEN 
		SELECT pais INTO ps_cont FROM tb_Contadores WHERE id_contador=id_cont;
	END IF;

        
	--No se han producido excepciones por lo que realizamos la modificacin
	UPDATE tb_Contadores SET direccion=dir_cont, localidad=loc_cont, pais=ps_cont WHERE id_contador=id_cont;

        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
   	
	--Ejecutamos commit para confirmar la modificacin de los datos
	COMMIT;
	
    	--Tratamiento de excepciones

	EXCEPTION
          WHEN EX_CONTADOR_NO_EXISTE THEN 
            RSP:='ERROR: PRMCO001: El contador '|| id_cont ||' NO ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
                    
          WHEN EX_VALORES_NULOS THEN
             RSP:='ERROR: PRMCO002: No se ha pasado ningn valor que modifique el contador';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
                  
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
      END;
