--Modificacin contrato--
/*Procedimiento que a partir de los parmetros de entrada aade un nuevo contrato a la base de datos
 *PARAM IN: contrato (variable de tipo Varchar2, contiene el identificador nico del contrato)
 *	    baja (variable de tipo DATE, contiene la fecha de alta del contrato)
 *	    potencia_contratada (variable de tipo NUMBER, contiene la potencia contratada por el cliente)
 *	    RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_modifica_contrato(contrato IN VARCHAR2, baja IN DATE,potencia_cont IN NUMBER, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;

   fecha_a DATE;
   pot_cont NUMBER(5,2):=potencia_cont;
    --Variable de comprobacion
    existe_contrato INT;
    contrato_activo INT;

    --Declaracin de excepciones
    EX_CONTRATO_NO_EXISTE EXCEPTION;
    EX_BAJA_MENOR_ALTA EXCEPTION;
    EX_VALORES_NULOS EXCEPTION;
    EX_CONTRATO_EN_BAJA EXCEPTION;
    
    --Ejecucin del procedimiento.
    BEGIN
        
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_contrato';

        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (contrato||'  ||  ' ||TO_CHAR(baja,'DD-MM-YY')||'  ||  '  || potencia_cont); 
       
        --Comprobamos que el contrato existe y que no ha sido dado de baja
        SELECT COUNT(*)INTO existe_contrato FROM tb_Contratos c WHERE c.id_contrato=contrato;
        SELECT COUNT(*) INTO contrato_activo FROM tb_Contratos co WHERE co.id_contrato=contrato AND fecha_baja IS NOT NULL;
        IF(existe_contrato= 0) THEN RAISE EX_CONTRATO_NO_EXISTE; END IF;
        IF(contrato_activo > 0) THEN RAISE EX_CONTRATO_EN_BAJA; END IF;
        
        SELECT fecha_alta INTO fecha_a FROM tb_Contratos co WHERE co.id_contrato=contrato;
        IF(baja IS NOT NULL AND fecha_a > baja) THEN RAISE EX_BAJA_MENOR_ALTA;END IF;
        --Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF ((baja is NULL) AND (potencia_cont is NULL) ) 
                THEN RAISE EX_VALORES_NULOS;
        ELSE
                IF(potencia_cont is NULL)THEN SELECT potencia_contratada INTO pot_cont FROM tb_Contratos co WHERE id_contrato=contrato;END IF;
        END IF;
      
            
        --No se existen excepciones por lo que realizamos las inserciones
        UPDATE tb_Contratos SET fecha_baja=baja, potencia_contratada=pot_cont WHERE id_contrato=contrato;
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	
	--Ejecutamos commit para confirmar la inserccin definitiva en la BD.
	COMMIT;   

    --Tratamiento de excepciones
   
    EXCEPTION
          WHEN EX_CONTRATO_NO_EXISTE THEN 
            RSP:='ERROR: PRMCN001: El contrato '|| contrato ||' NO ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
          
          WHEN EX_VALORES_NULOS THEN
             RSP:='ERROR: PRMCN002: no se han pasado valores para realizar la modificacin';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
                  
          WHEN EX_CONTRATO_EN_BAJA THEN 
            RSP:='ERROR: PRMCN003: El contrato '||contrato||' ya tiene fecha de baja';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);                    
          COMMIT; 

          WHEN EX_BAJA_MENOR_ALTA THEN 
            RSP:='ERROR: PRMCN004: La fecha de baja del contrato '||contrato||' no puede ser menor que la fecha de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;

            
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;

      END;
