/*Procedimiento que a partir de la modificacin de la lectura del consumo de los contadores  permite modificar la
 *produccin de una Central de produccin en funcin de este. Se ha supuesto un incremento en la
 *produccin de Central de un 25 % sobre el consumo de los contadores. Para determinar el consumo
 *de cada central se ha repartido el consumo de cada contador entre las Centrales de distribucin
 *y el resultado entre la lneras de alimentacin. Adems se modifican tambin algunas de las tablas que componen
 *el mdulo estadstico.
 *PARAMETROS IN: id_cont(Variable de tipo NUMBER, contiene el identificador del contador)
 *               fecha_lect (Variable de tipo DATE, contiene la fecha de la lectura del contador)
 *               consumo_new (Variable de tipo NUMBER, contiene el nuevo consumo mensual del contador)
 *		 consumo_old (Variable de tipo NUMBER, contiene el consumo del contador antes de actualizar)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_modifica_historico(id_cont IN NUMBER, fecha_lect IN DATE, consumo_new IN NUMBER, consumo_old IN NUMBER, RSP OUT NOCOPY CLOB) AS

    --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;

    --Declaracin de variables 
    mes_lectura INTEGER;
    anyo_lectura INTEGER;
    id_central_p VARCHAR2(10);
    cod_linea NUMBER(10);
    maxima_produccion NUMBER (10);
    distribucion_energia NUMBER(10,2);
    produccion_new NUMBER(8);
    produccion_old NUMBER(8);
    new_consumo_centrales NUMBER(10,2);
    old_consumo_centrales NUMBER(10,2);
    produccion_acumulada NUMBER (10);
    num_centrales INTEGER;
    num_lineas INTEGER;
    num_lineas_por INTEGER;
         --Variables mdulo estadstico, apartado 1,3 y 4
    consumo_lectura NUMBER(10,2);
    consumo_centrales NUMBER(10,2);
    consumo_linea NUMBER(10,2);
    consumo_acumulado_linea NUMBER(16,2);
    num_total_lineas INTEGER;
    porcentaje NUMBER(3);
    porcentaje_lineas NUMBER(5,2);
        --Variables mdulo estadistico, apartado 2.
    num_contadores INTEGER;
    consumo_medio NUMBER(10,2);
    consumo_medio_acumulado NUMBER(16,2);
     
        --Variables mdulo estadstico 5
    num_centrales_p NUMBER(10);
    new_porcentaje NUMBER(5,2);
    existe_consumo INTEGER;
    produccion_acumulada_h NUMBER (10);
    cod_central_p_aux VARCHAR2(10);
    produccion_central_h NUMBER(10);
    
    --Variables mdulo estadstico, apartado 6 y 7.
    cod_contador NUMBER(10);
    consumo_acumulado_contador NUMBER(16,2);
    num_clientes INTEGER;

    --Variables para identificar la central de distribucion
    dir VARCHAR2(50);
    loc VARCHAR2(50);
    ps VARCHAR2(25);
    
    --Declaracion de cursor
        --Declaracion de cursores
    CENTRALES_P SYS_REFCURSOR;
    CURSOR_AUXILIAR SYS_REFCURSOR;
    CURSOR_AUXILIAR_DOS SYS_REFCURSOR;

    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_historico';
      
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (id_cont||'  ||  ' || fecha_lect ||'  ||  ' || consumo_new||' || '|| consumo_old);  
	
	--Obtenemos los valores abolutos a partir de los consumos  

	produccion_new:=ABS(consumo_new +(consumo_new*25/100));
        produccion_old:=ABS(consumo_old +(consumo_old*25/100));

      	--Dividimos la producciones absolutas y los consumos entre el nmero de centrales de distribucin que suministran al contador

	SELECT COUNT(*) INTO num_centrales FROM tb_Distribucion td WHERE td.contador=id_cont;
        produccion_new:=produccion_new/ num_centrales;
        produccion_old:=produccion_old/ num_centrales;
	new_consumo_centrales:=consumo_new/num_centrales;
	old_consumo_centrales:=consumo_old/num_centrales;
	
	--Obtenemos el mes y el ao de la fecha de la lectura
	SELECT EXTRACT(MONTH FROM fecha_lect)INTO mes_lectura FROM DUAL;
	SELECT EXTRACT(YEAR FROM fecha_lect)INTO anyo_lectura FROM DUAL;
                
       --Abrimos el cursor
       OPEN CENTRALES_P FOR
            SELECT tp.codigo_central, ta.direccion, ta.localidad, ta.pais, ta.id_linea FROM TB_CENTRALES_P tp INNER JOIN (TB_LINEAS tl INNER JOIN 
            (TB_ALIMENTACION ta INNER JOIN TB_DISTRIBUCION td ON  ta.direccion=td.direccion AND ta.localidad=td.localidad AND ta.pais=td.pais)
            ON codigo_linea=id_linea) ON codigo_central=central_p 
            WHERE contador=id_cont;
        
        --Recorremos el cursor.  
        LOOP FETCH CENTRALES_P INTO id_central_p,dir,loc,ps,cod_linea;
        
        --Aseguramos la terminacin del bucle cuando no haya ms filas.
        EXIT WHEN CENTRALES_P%NOTFOUND;          
           --Estamos modificando una lectura anterior por lo que ya existe un histrico de produccin. Adems no se ha modificado
           --ni el id del contador ni la fecha de la lectura.
            
	   --Modificamos el total de energia emitida por la central de distribucin
	    SELECT energia_emitida INTO distribucion_energia FROM tb_Centrales_D tcd WHERE tcd.direccion=dir AND tcd.localidad=loc AND tcd.pais=ps;
	   distribucion_energia:=distribucion_energia+new_consumo_centrales-old_consumo_centrales;
	   UPDATE tb_Centrales_D tcd SET energia_emitida=distribucion_energia WHERE tcd.direccion=dir AND tcd.localidad=loc AND tcd.pais=ps;	

            --Determinamos el nmero de lneas usado por cada central de distribucin para repartir el consumo entre ellas.
            SELECT COUNT(*) INTO num_lineas FROM tb_Alimentacion ta WHERE ta.direccion=dir AND ta.localidad=loc AND ta.pais=ps;
	               
            --Modificamos la produccin               
                SELECT produccion INTO produccion_acumulada FROM tb_Historico th WHERE th.mes=mes_lectura AND th.anyo=anyo_lectura AND th.central_p=id_central_p;
                produccion_acumulada:=(produccion_new/num_lineas) + produccion_acumulada-(produccion_old/num_lineas);
                UPDATE tb_Historico SET produccion=produccion_acumulada WHERE mes=mes_lectura AND anyo=anyo_lectura AND central_p=id_central_p;
            --Comprobamos si la produccin del mes supera a la produccin mxima de la central en un mes
            SELECT maxima_produccion_mes INTO maxima_produccion FROM tb_Centrales_P WHERE codigo_central=id_central_p;
              
            IF(maxima_produccion < produccion_acumulada)THEN 
                  UPDATE tb_Centrales_P SET maxima_produccion_mes=produccion_acumulada WHERE codigo_central=id_central_p;
            END IF;

	    /*Mdulo estadstico, apartado 1 
	     *Establecemos el consumo de los contadores conectados a una central  de produccin a partir del consumo de la lectur
	     *del mes.		     
             */

	    	--Obtenemos el valor del consumo acumulado, le aadimos el nuevo consumo y le restamos el consumo anterior.
             SELECT consumo_acumulado INTO consumo_lectura FROM me_Central_consumo_contadores WHERE codigo_central_p=id_central_p;
             consumo_lectura:=consumo_lectura+((new_consumo_centrales-old_consumo_centrales)/num_lineas);
             UPDATE me_Central_consumo_contadores SET consumo_acumulado=consumo_lectura WHERE codigo_central_p=id_central_p;  
 
            /*Fin Mdulo estadstico, apartado 1 */	
            
            /* Mdulo estadstico, apartado 2 
             * Se ha de tener en cuenta que el consumo de cada contador se ha dividido entre las distintas centrales
             * de distribucin a las que estn conectados, sin embargo cada una de las lineas conectadas a
             * las centrales de distribucin computa el consumo de la central como si fuera la nica lnea que la alimentara.
             */
            
		--Obtenemos el nmero de contadores alimentados por la lnea actual
	    SELECT COUNT(DISTINCT td.contador)INTO num_contadores FROM tb_Alimentacion tal 
            INNER JOIN tb_distribucion td ON tal.direccion=td.direccion AND tal.localidad=td.localidad AND tal.pais=td.pais
            WHERE tal.id_linea=cod_linea;

		--Calculamos el consumo medio a modificar en funcin del consumo nuevo-el antiguo y del nmero de contadores
		--alimentados por la lnea actual.
            consumo_medio:=(consumo_new-consumo_old)/num_contadores;

           	--Obtenemos el consumo medio de la tabla estadstica, le aadimos el nuevo consumo medio y actualizamos 
	    SElECT valor_medio_consumo INTO consumo_medio_acumulado FROM me_Valor_medio_consumo WHERE id_linea=cod_linea AND anyo=anyo_lectura;
            consumo_medio_acumulado:=consumo_medio_acumulado+consumo_medio;
            UPDATE me_Valor_medio_consumo SET valor_medio_consumo=consumo_medio_acumulado WHERE id_linea=cod_linea AND anyo=anyo_lectura;
            /*Fin mdulo estadstico, apartado 2 */
            
           
	    /*Mdulo estadstico, apartado 3 
             *Como en el apartado anterior y siguiendo las indicaciones del cliente,  se ha computado el consumo 
             *de cada lnea como si fuera la nica que alimenta a la central de distribucin.
             */

		--Comprobamos si la lnea actual ha estado ms cargada que la lnea incorporada a la tabla estadstica
		--en funcin de la modificacin del consumo.

		--Obtenemos el consumo de la lnea que actualmente ha estado ms cargada.
             SELECT max_total_consumo INTO consumo_lectura FROM me_Linea_mas_cargada;

		--Obtenemos el consumo de la lnea actual
             SELECT SUM(NVL(energia_emitida/num_lineas,0))INTO consumo_acumulado_linea FROM tb_Centrales_D tcd
             INNER JOIN tb_Alimentacion tal ON tal.direccion=tcd.direccion AND tal.localidad=tcd.localidad AND tal.pais=tcd.pais
             WHERE tal.id_linea=cod_linea;

		--Comprobamos si la lnea actual supera el consumo de la lnea incorporada a la tabla estadstica, si es asi
		--actualizamos.
             IF(consumo_lectura < consumo_acumulado_linea) THEN
                      UPDATE me_Linea_mas_cargada SET id_linea=cod_linea, max_total_consumo=consumo_acumulado_linea WHERE identificador=1;
             END IF;
             /*Fin mdulo estadstico, apartado 3 */
                          
         
        END LOOP;
        CLOSE CENTRALES_P;
         
        /*Mdulo estadstico 5 
        *Se ha modificado el histrico por lo que recalculamos el nmero de centrales de produccin que producen 
        *menos del 30% de la produccin total de un ao dado
        */
        
        --Limpiamos la tabla
        DELETE FROM me_numero_centrales_menos_30 WHERE anyo=anyo_lectura;
        num_centrales_p:=0;
        
        --Recorremos la tabla de los historicos de produccin para calcular lo solicitado.
        OPEN CURSOR_AUXILIAR_DOS FOR
             SELECT central_p  FROM tb_Historico WHERE anyo=anyo_lectura
             GROUP BY central_p;
             --Recorremos el cursor.  
             LOOP FETCH CURSOR_AUXILIAR_DOS INTO cod_central_p_aux;
             --Aseguramos la terminacin del bucle cuando no haya ms filas.
             EXIT WHEN CURSOR_AUXILIAR_DOS%NOTFOUND;  

			--Obtenemos el total de la produccin del ao de la lectura
                    SELECT SUM(produccion)INTO produccion_acumulada_h FROM tb_Historico WHERE anyo=anyo_lectura;

			--Obtenemos el total de la produccin de la central de produccin en el ao de la lectura.
                    SELECT SUM(produccion)INTO produccion_central_h FROM tb_Historico WHERE anyo=anyo_lectura AND central_p=cod_central_p_aux;

			--Obtenemos el porcentaje de produccin y comprobamos si es menor del 30%
                    new_porcentaje:=produccion_central_h*100/produccion_acumulada_h;
                    IF(new_porcentaje <30)THEN
                          num_centrales_p:=num_centrales_p+1;
                    ELSE 
                          num_centrales_p:=num_centrales_p+0;
                    END IF;

			--Comprobamos que ya existen datos introducidos en la tabla estadstica para el ao de la lectura modificada
                    SELECT COUNT(*)INTO existe_consumo FROM me_numero_centrales_menos_30 WHERE anyo=anyo_lectura;

			--Sin no existen aadimos los resultados obtenidos
                    IF(existe_consumo=0)THEN
                          INSERT INTO me_numero_centrales_menos_30 VALUES (anyo_lectura, num_centrales_p);

			--Si ya existen modifiamos en funcin de los resultados obtenidos.
                    ELSE
                          UPDATE me_numero_centrales_menos_30 SET num_centrales=num_centrales_p WHERE anyo=anyo_lectura;
                    END IF;
             END LOOP;
        CLOSE CURSOR_AUXILIAR_DOS;
        /*Fin mdulo estadstico 5*/  

        /*Mdulo estadstico, apartado 6
	 *Top de los 10 contadores que han tenido ms consumo historicamente.
	 *
         *Solo actualizaremos este mdulo si la lectura nueva lectura fuera mayor que la lectura
         *introducida por error  y a la que modifica
         */
        
        IF(consumo_new > consumo_old) THEN 

		--Obtenemos el valor del consumo acumulado del contador que ocupa la 10 posicin en el TOP.
            SELECT MIN(consumo_acumulado)INTO consumo_lectura FROM
                (SELECT consumo_acumulado FROM tb_Contadores ORDER BY consumo_acumulado DESC)
            WHERE ROWNUM <=10;
      
		--Actualizamos el valor del consumo_acumulado del contador actual, puesto que este an no ha sido
		--modificado en la tabla TB_CONTADORES.	
            SELECT consumo_acumulado INTO consumo_acumulado_contador FROM tb_contadores WHERE id_contador=id_cont;
            consumo_acumulado_contador:=consumo_acumulado_contador+consumo_new-consumo_old;
        
            	--Comprobamos si el contador ya est incluido en el TOP
	    SELECT COUNT(*)INTO existe_consumo FROM me_top_contadores_mas_consumo WHERE id_contador=id_cont;
            
		--Obtenemos el nmero de contadores incorporados al TOP
	    SELECT COUNT(*)INTO num_contadores FROM me_top_contadores_mas_consumo;

		--Si el consumo acumulado del contador es mayor que el consumo del contador con mnimo consumo en el TOP
		--lo introducimos en la tabla estadstica, dependiendo de si ya existe en la tabla y de si el nmero de
		--contadores de esta es igual a 10.
            IF((existe_consumo=0) AND (consumo_acumulado_contador > consumo_lectura)) THEN
                IF(num_contadores=10)  THEN
                      DELETE FROM me_top_contadores_mas_consumo 
                      WHERE id_contador =(SELECT id_contador FROM tb_contadores WHERE consumo_acumulado=consumo_lectura);
                END IF;
                INSERT INTO me_top_contadores_mas_consumo VALUES(id_cont,consumo_acumulado_contador);
            ELSE
                UPDATE me_top_contadores_mas_consumo SET consumo=consumo_acumulado_contador WHERE id_contador=id_cont; 
            END IF;
        END IF;  

	--Utilizamos un cursor auxiliar para ordenar la tabla 
        --por el consumo en orden descendente
        OPEN CURSOR_AUXILIAR FOR
              SELECT id_contador, consumo FROM me_top_contadores_mas_consumo ORDER BY consumo DESC;
              --Recorremos el cursor y ordenamos la tabla.  
              LOOP FETCH CURSOR_AUXILIAR INTO cod_contador, consumo_acumulado_contador;
              --Aseguramos la terminacin del bucle cuando no haya ms filas.
              EXIT WHEN CURSOR_AUXILIAR%NOTFOUND;  
                      DELETE FROM me_top_contadores_mas_consumo WHERE id_contador=cod_contador;
                      INSERT INTO me_top_contadores_mas_consumo VALUES(cod_contador,consumo_acumulado_contador);
              END LOOP;
        CLOSE CURSOR_AUXILIAR;
        /*Fin mdulo estadstico, apartado 6*/
        
        /*Modulo estadstico, apartado 7
	 *Como en el caso anterio solo actualizamos la tabla en el supuesto de que el nuevo consumo sea
	 *mayor que el viejo consumo
	 */
        IF(consumo_new > consumo_old) THEN 

		--Obtenemos el nmero total de clientes.
                SELECT COUNT(*)INTO num_clientes FROM tb_Clientes;

		--Obtenemos la suma del consumo de todos los contadores y actualizamos la tabla estadstica.
                SELECT SUM(CONSUMO_ACUMULADO) INTO consumo_acumulado_contador FROM tb_Contadores;
                UPDATE me_consumo_medio_clientes mcmc SET mcmc.consumo_medio=(consumo_acumulado_contador+consumo_new-consumo_old)/num_clientes;
        END IF;
        /*Fin mdulo estadstico, apartado 7*/
        
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP); 
        
     --Los posibles errores fueron capturados el modificar la lectura por lo que capturamos aquellos que no hemos previsto.   
     EXCEPTION
     
         WHEN OTHERS THEN 
         RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
         INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;


