--Modificacin lectura

/*Procedimiento que a partir de los parmetros de entrada modifica una lectura en la base de datos, adems
 *de actualizar la tabla de mdulo estadstico correspondiente al porcentaje de las lneas que superan el 
*50% de la energa consumida.
 *PARAMETROS IN: id_conta (Variable de tipo NUMBER, contiene el identificador nico del contador)
 *		 fecha_lect (Variable de tipo DATE, contiene la fecha de la lectura del contador)
 *		 modalidad_lectura (Variable de tipo Varchar2, contiene el tipo de lectura (TELEMATICA o PRESENCIAL).
 *		 lectura (Variable de tipo NUMBER, contiene el valor de la lectura del contador) 
 *	         RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */

CREATE OR REPLACE
PROCEDURE pr_modifica_lectura(id_conta IN NUMBER, fecha_lect IN DATE, modalidad_lectura IN VARCHAR, lectura IN NUMBER, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;
    
     
    consumo_mensual NUMBER(6,2);
    con_mes NUMBER(6,2);
    mod_lectura VARCHAR2(15):=modalidad_lectura;
    lectura_mes NUMBER:=lectura;
    consumo_anterior NUMBER(10,2);
    consumo_acumulado_linea NUMBER(16,2);
    consumo_lectura NUMBER(10,2);
    num_total_lineas INTEGER;
    num_lineas_por INTEGER;
    porcentaje INTEGER;
    porcentaje_lineas NUMBER(5,2);
	    
    --Variables 
    num_centrales INTEGER;
    anyo_lectura NUMBER(4);    
    cod_linea NUMBER(10);

    --Variables de comprobacion
    existe_lectura INT;
   
    --Declaracin de excepciones
    EX_LECTURA_NO_EXISTE EXCEPTION;
    EX_VALORES_NULOS EXCEPTION;

    --Declaracin de Cursor	
    me_lineas_50 SYS_REFCURSOR;
    --Ejecucin del procedimiento.
    BEGIN
 
        --Obtenemos la fecha de ejecucin del procedimiento de los datos del sistema.
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modifica_lectura';

	--Creamos una cadena de texto con los parametros de entrada.
        In_log:= (id_conta||'  ||  ' ||TO_CHAR(fecha_lect,'DD-MM-YYYY')||'  ||  ' ||modalidad_lectura||'  ||  ' || lectura); 

	--Obtenemos el ao de la lectura
	SELECT EXTRACT(YEAR FROM fecha_lect)INTO anyo_lectura FROM DUAL;

	--Obtenemos el nmero de centrales que distribuyen al contador
   	SELECT COUNT(*) INTO num_centrales FROM tb_Distribucion td WHERE td.contador=id_conta;
               
        --Comprobamos si la lectura ya ha sido dada de alta en la base de datos
	SELECT COUNT(*)INTO existe_lectura FROM tb_Lecturas lec WHERE (lec.contador=id_conta AND lec.fecha=fecha_lect);
        IF(existe_lectura = 0) THEN RAISE EX_LECTURA_NO_EXISTE; END IF;
  
        IF(lectura IS NULL AND modalidad_lectura IS NULL) THEN RAISE EX_VALORES_NULOS; END IF;

	--Guardamos el valor del consumo mensula anterior.
	SELECT consumo_mes INTO consumo_mensual FROM tb_lecturas lec WHERE (lec.contador=id_conta AND lec.fecha=fecha_lect);
        IF(lectura is NULL) THEN
            SELECT lectura_total INTO lectura_mes FROM tb_Lecturas lec WHERE (lec.contador=id_conta AND lec.fecha=fecha_lect);
        END IF;
        
        IF(modalidad_lectura is NULL)THEN
            SELECT modalidad INTO mod_lectura FROM tb_Lecturas lec WHERE (lec.contador=id_conta AND lec.fecha=fecha_lect);
      	END IF;

	--Comprobamos si la lectura nueva lectura es inferior a la lectura anterior
	--De ser asi el valor del consumo mensual sera igual a 0
	SELECT consumo_acumulado INTO consumo_anterior FROM tb_Contadores WHERE id_contador=id_conta;
	IF(consumo_anterior <= lectura_mes) THEN
	        SELECT consumo_mes INTO con_mes FROM tb_Lecturas lec WHERE (lec.contador=id_conta AND lec.fecha=fecha_lect);
        	consumo_mensual:=lectura_mes-(consumo_anterior-con_mes);
        	--actualizamos el consumo acumulado en el contador SOLO si el valor de la nueva lectura es superior al pasado
		--por ERROR				
            	UPDATE tb_Contadores con SET con.consumo_acumulado=lectura_mes WHERE con.id_contador=id_conta;
        ELSE
                consumo_mensual:=0.0;
	END IF;

	/*Mdulo estadstico, apartado 4 
             /*El consumo de las lneas esta calculado suponiendo que estas computan el consumo de cada una de las 
              *centrales de distribucin a las que alimentan como si fuera la nica linea.
              */     	

	--Abrimos el cursor
        OPEN me_lineas_50 FOR
            SELECT ta.id_linea FROM TB_LINEAS tl INNER JOIN 
            (TB_ALIMENTACION ta INNER JOIN TB_DISTRIBUCION td ON  ta.direccion=td.direccion AND ta.localidad=td.localidad AND ta.pais=td.pais)
            ON codigo_linea=id_linea
            WHERE contador=id_conta;
        
        --Recorremos el cursor.  
        LOOP FETCH me_lineas_50 INTO cod_linea;
                
        --Aseguramos la terminacin del bucle cuando no haya ms filas.
        EXIT WHEN me_lineas_50%NOTFOUND;  
        
             SELECT SUM(consumo_mes/2)INTO consumo_lectura FROM tb_Lecturas tle WHERE anyo_lectura =(SELECT EXTRACT(YEAR FROM tle.fecha)FROM DUAL);
             consumo_lectura:=consumo_lectura+((lectura-consumo_anterior)/2);

             SELECT COUNT(*)INTO num_total_lineas FROM tb_Lineas;
             SELECT SUM(consumo_mes)INTO consumo_acumulado_linea 
                        FROM tb_Lecturas tle INNER JOIN(tb_Distribucion tdi INNER JOIN tb_Alimentacion tal
                        ON tdi.direccion=tal.direccion AND tdi.localidad=tal.localidad AND tdi.pais=tal.pais) ON tle.contador=tdi.contador
                        WHERE tal.id_linea=cod_linea AND anyo_lectura=(SELECT EXTRACT(YEAR FROM tle.fecha)FROM DUAL);

                        consumo_acumulado_linea:=((lectura-consumo_anterior)/num_centrales);

                        IF(consumo_acumulado_linea > consumo_lectura)THEN porcentaje:=1;
                        ELSE porcentaje:=0;
                        END IF;
                        consumo_acumulado_linea:=((consumo_acumulado_linea+lectura-consumo_anterior)/num_centrales);
                         
                        --Comprobamos que la lnea supera el 50% del consumo y que no estaba ya incluida en el porcentaje
                        --de ser asi, la aadimos
                        IF (consumo_acumulado_linea > consumo_lectura AND porcentaje=0) THEN
                                      SELECT ROUND(
                                      (SELECT PORCENTAJE_MAS_50 FROM me_porcentaje_lineas_mas_50 WHERE anyo=anyo_lectura)*num_total_lineas/100,0)
                                      INTO num_lineas_por FROM DUAL;
                                      num_lineas_por:=num_lineas_por+1;
                        END IF;
                        --Comprobamos si la lnea supera el 50% del consumo, si no lo superara y estuviera incluida
                        --en el porcentaje anterior la eliminamos.
                        IF (consumo_acumulado_linea < consumo_lectura AND porcentaje=1) THEN
                                     SELECT ROUND(
                                      (SELECT PORCENTAJE_MAS_50 FROM me_porcentaje_lineas_mas_50 WHERE anyo=anyo_lectura)*num_total_lineas/100,0)
                                      INTO num_lineas_por FROM DUAL;
                                      num_lineas_por:=num_lineas_por-1;
                          END IF;
                          
                          --Recalculamos el porcentaje de lneas.
                          IF(num_lineas_por IS NULL) THEN 
                                porcentaje_lineas:=0;
                          ELSE
                                porcentaje_lineas:=num_lineas_por*100/num_total_lineas;
                         END IF;
                         UPDATE me_porcentaje_lineas_mas_50 SET porcentaje_mas_50=porcentaje_lineas WHERE anyo=anyo_lectura;
             /*Fin mdulo estadstico 4 */
	     END LOOP;
	     CLOSE me_lineas_50;
         
        --No existen excepciones por lo que realizamos la acturalizacin
        UPDATE tb_Lecturas  SET modalidad=mod_lectura ,lectura_total=lectura_mes, consumo_mes=consumo_mensual WHERE(contador=id_conta AND fecha=fecha_lect);
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--ejecutamos commit para confirmar la inserccin definitiva en la BD.
	COMMIT;   

   --Tratamiento de excepciones

    EXCEPTION
          WHEN EX_LECTURA_NO_EXISTE THEN 
            RSP:='ERROR: PRMLC001: La lectura del contador '|| id_conta ||' en fecha ' ||fecha_lect|| ' NO ha  sido introducida en la base de datos';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
          
          WHEN EX_VALORES_NULOS THEN
             RSP:='ERROR: PRMLC002: No se han pasado valores para modificar';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;          


          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
      END;
