--Modificacin Lnea--
/*Procedimiento que a partir de los parmetros de entrada modifica la capacidad de una lnea de la base de datos
 *PARAMETROS IN: codigo (Vairabe de tipo NUMBER, contiene la identificacion de la lnea)
 *		 capacidad_linea (Variable de tipo NUMBER, contiene la capacidad mximad de energa que la lnea puede transportar)
 *		 RSP (RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno). 	    
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_modificacion_linea(id_linea IN NUMBER, capacidad_linea IN NUMBER, RSP OUT NOCOPY CLOB) AS
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(30);
    In_log CLOB;
    
    --variables de comprobacion
    existe_linea INTEGER;

    --Declaracin de excepciones
    EX_LINEA_NO_EXISTE EXCEPTION;
    EX_LINEA_CAP_NULL EXCEPTION;
     
    --Ejecucin del procedimiento.
    BEGIN
          
        --Obtenemos la fecha del ejecucin de procedimiento de los datos del sistema
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_modificacion_linea';
      
        --Creamos una cadena de texto con los parametros de entrada separados por barras.
        In_log:= (id_linea||'  ||  ' || capacidad_linea ); 
       
        --Comprobamos que existe la lnea 
        
        SELECT COUNT(*)INTO existe_linea FROM tb_Lineas lin WHERE lin.codigo_linea=id_linea;
        
        --Si no existe capturamos la excepcin para su posterior tratamiento.
        IF(existe_linea = 0) THEN RAISE EX_LINEA_NO_EXISTE; END IF;
        
        --Comprobamos que la capacidad_linea tiene algn valor
        IF(capacidad_linea is NULL)THEN RAISE EX_LINEA_CAP_NULL; END IF;
        
        --Si no se han producido excepciones realizamos la inserccin.    
        UPDATE tb_Lineas SET capacidad=capacidad_linea WHERE codigo_linea=id_linea;
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--Ejecutamos commit para confirmar la modificacin la BD
	COMMIT;        

    --Tratamiento de excepciones
    EXCEPTION
          WHEN EX_LINEA_NO_EXISTE THEN 
            RSP:='ERROR: PRMLI001: La lnea con el cdigo '|| id_linea ||' NO ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT; 

          WHEN EX_LINEA_CAP_NULL THEN 
            RSP:='ERROR: PRMLI002: La capacidad de la lnea n '||id_linea||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;             
        
        
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;       
END;
