/*Procedimiento que a partir de la lectura del consumo de los contadores  nos permitir determinar
 *los contadores cuyo consumo mensual supera el 80% del consumo medio de todos los contadores de una
 *ciudad dada en un mes determinado
 *PARAMETROS IN: ciudad(Variable de tipo VARCHAR2, contiene la ciudad sobre la que realizaremos la consulta)
 *               fecha_consulta (Variable de tipo DATE, cotiene la fecha a partir de la cual realizaremos la consulta)
 *               CURSOR_R (Variable de tipo SYS_REFCURSOR, que se corresponde con el parmetro de retorno que contendr
 *			   los datos solicitados)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: 	 CURSOR_R (Contendr la respuesta con el Cdigo del contrato , la potencia y el porcentaje con respecto al 
 *		 	   consumo medio de los contadores de la Ciudad, segn las especificaciones inciales).
 *		 Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */


CREATE OR REPLACE
PROCEDURE pr_consulta_a(ciudad IN VARCHAR2, fecha_consulta IN DATE, CURSOR_R OUT NOCOPY SYS_REFCURSOR, RSP OUT NOCOPY CLOB) AS

    --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(35);
    In_log CLOB;

    --Declaracin de variables 
    
    consumo_medio NUMBER(6,2);
    id_contrato VARCHAR2(15);
    id_potencia NUMBER(5,2);
    porcent NUMBER(3);
    primer_dia_mes DATE;
    ultimo_dia_mes DATE;
    
    --variable de comprobacin
    existe_ciudad INTEGER;
    existe_lectura INTEGER;
    
    --Excepciones
    EX_CIUDAD_NO_EXISTE EXCEPTION;
    EX_NO_EXISTE_CONSUMO_MES EXCEPTION;
    
    --Iniciamos
    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_consulta_a';
      
        --Creamos una cadena de texto con los parametros de entrada.

        In_log:= (ciudad||'  ||  ' || fecha_consulta);  
        

	--Obtenermos el primer y ultimo da del mes a partir del parmetro fecha_consulta 
	SELECT TRUNC(fecha_consulta,'MM')INTO primer_dia_mes FROM DUAL;
	SELECT LAST_DAY(fecha_consulta)INTO ultimo_dia_mes FROM DUAL;	
        
        --Comprobamos que existen contadores en la ciudad pasada como parametro
        SELECT COUNT(*)INTO existe_ciudad FROM tb_contadores WHERE localidad=ciudad;
        
        --Comprobamos que existen lecturas en el mes de la consulta.
        
        SELECT COUNT(*)INTO existe_lectura FROM tb_Lecturas tl INNER JOIN tb_Contadores tc
        ON tl.contador=tc.id_contador
        WHERE tc.localidad=ciudad 
        AND tl.fecha BETWEEN TO_DATE(primer_dia_mes) AND TO_DATE( ultimo_dia_mes);
        
        IF(existe_ciudad=0)THEN RAISE EX_CIUDAD_NO_EXISTE; END IF;
        IF(existe_lectura=0)THEN RAISE EX_NO_EXISTE_CONSUMO_MES; END IF;

	--Calculamos la media aritmtica del consumo de los contadores de la ciudad durante el mes y el ao de consulta
 
	SELECT AVG(tl.consumo_mes)INTO consumo_medio FROM 
	tb_Lecturas tl INNER JOIN tb_Contadores tc ON tl.contador=tc.id_contador 
	WHERE tc.localidad=ciudad 
	AND tl.fecha BETWEEN TO_DATE(primer_dia_mes) AND TO_DATE( ultimo_dia_mes);
                
       --Abrimos el cursor
       
        OPEN CURSOR_R FOR
                  SELECT id_contrato, potencia_contratada, (consumo_mes*100/consumo_medio)AS porcentaje
		  FROM tb_Lecturas tl INNER JOIN (tb_contadores tc INNER JOIN tb_contratos tcon ON tc.id_contador=tcon.contador)
		  ON tl.contador=tc.id_contador
	    	  WHERE tc.localidad=ciudad 
	          AND tl.fecha BETWEEN TO_DATE(primer_dia_mes) AND TO_DATE(ultimo_dia_mes)
		  AND (consumo_mes >(consumo_medio * 0.80))
		  ORDER BY porcentaje;
        
        --Recorremos el cursor para comprobar resultados 
        LOOP FETCH CURSOR_R INTO id_contrato, id_potencia, porcent;
        EXIT WHEN CURSOR_R%NOTFOUND;  
		dbms_output.put_line(id_contrato ||'   '||id_potencia||'    '||porcent);
        END LOOP;
	        
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,'cursor con los datos solicitados',RSP); 
        COMMIT;
 
     EXCEPTION
      
        WHEN EX_CIUDAD_NO_EXISTE THEN
        RSP:='ERROR: PRCOA001: La ciudad '||ciudad||' no existe o no tiene contadores registrados, verifique el nombre';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
        
        WHEN EX_NO_EXISTE_CONSUMO_MES THEN
        RSP:='ERROR: PRCOA002: Los contadores de la ciudad '||ciudad||' no tienen lectura registrada en el mes y ao solicitados';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
        
        WHEN OTHERS THEN 
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;

