/*Procedimiento que nos devuelve el listado de las 10 centrales de distribucin que han emitido ms energia.
 *PARAMETROS IN: CURSOR_R (Variable de tipo SYS_REFCURSOR, que se corresponde con el parmetro de retorno que contendr
 *			   los datos solicitados)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: 	 CURSOR_R (Contendr la direccin, la localidad, el pais, la energa emitida y el suministro mximo de las centrales
 *			    de distribucin, mostrado en este orden).
 *		 Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */


CREATE OR REPLACE
PROCEDURE pr_consulta_b(CURSOR_R OUT NOCOPY SYS_REFCURSOR, RSP OUT NOCOPY CLOB) AS

    --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(35);
    In_log CLOB;

    --Declaracin de variables 
    
    energia_emitida NUMBER(10,2);
    maximo_suministro NUMBER(8,2);
    direccion_central VARCHAR2(50);
    localidad_central VARCHAR2(50);
    pais_central VARCHAR2(25);
    
  
    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_consulta_b';
      
        --Creamos una cadena de texto con los parametros de entrada.

        In_log:= ('sin parmetros de entrada');  

                
       --Abrimos el cursor
       
        OPEN CURSOR_R FOR
                  SELECT * FROM 
			(SELECT tcd.direccion, tcd.localidad, tcd.pais, tcd.energia_emitida, tcd.suministro_maximo
			 FROM tb_Centrales_D tcd
			 ORDER BY tcd.energia_emitida DESC) 
		  WHERE ROWNUM BETWEEN 1 AND 10;


        --Recorremos el cursor para comprobar resultados a nivel particular, el cursor ser tratado por la aplicacin que
	--ha llamado al procedimiento desde donde deber ser cerrado para liberar los recursos
        LOOP FETCH CURSOR_R INTO direccion_central, localidad_central, pais_central, energia_emitida, maximo_suministro;
        EXIT WHEN CURSOR_R%NOTFOUND;  
		dbms_output.put_line(direccion_central ||'   '||localidad_central||'    '||pais_central||'    '||energia_emitida||'    '||maximo_suministro);
        END LOOP;
	        
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,'cursor con los datos solicitados',RSP); 
        COMMIT;
 
     EXCEPTION

        WHEN OTHERS THEN 
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;

