/*Procedimiento que nos devuelve el listado de las 10 lneas de alimentacin ms cargadas en funcin de su capacidad.
 *Se repartido el consumo de cada central de distribucin entre las lneas que la alimentan.
 *PARAMETROS IN: CURSOR_R (Variable de tipo SYS_REFCURSOR, que se corresponde con el parmetro de retorno que contendr
 *			   los datos solicitados)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: 	 CURSOR_R (Contendr la direccin, la localidad, el pais, la energa emitida y el suministro mximo de las centrales
 *			    de distribucin, mostrado en este orden).
 *		 Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */


CREATE OR REPLACE
PROCEDURE pr_consulta_c(CURSOR_R OUT NOCOPY SYS_REFCURSOR, RSP OUT NOCOPY CLOB) AS

    --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(35);
    In_log CLOB;

    --Declaracin de variables 
    codi_linea  NUMBER(10);
    ampliacion NUMBER(6);
    carga_linea NUMBER(10,2);
    
  
    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_consulta_c';
      
        --Creamos una cadena de texto con los parametros de entrada.

        In_log:= ('sin parmetros de entrada');  

                
       --Abrimos el cursor
       
        OPEN CURSOR_R FOR
	     SELECT * FROM (
		SELECT til.codigo_linea, SUM(
			tcd.energia_emitida*100/( 
				SELECT COUNT(*)FROM tb_alimentacion tl INNER JOIN tb_centrales_d td ON
				tl.direccion=td.direccion AND tl.localidad=td.localidad AND tl.pais=td.pais
				WHERE tcd.direccion=tl.direccion AND tcd.localidad=td.localidad AND tcd.pais=td.pais
				GROUP BY tl.direccion)/(
					SELECT tli.capacidad FROM tb_Lineas tli WHERE tli.codigo_linea=tal.id_linea)
				)AS Porcentaje_carga_linea,(
				CASE WHEN tp.tipo_central='SOLAR'THEN 1000
              			     WHEN tp.tipo_central='EOLICA'THEN 3000
                      		     WHEN tp.tipo_central='CARBON' THEN 2000
               			     WHEN tp.tipo_central='NUCLEAR'THEN 6000
      				     ELSE 4000
       				END) AS AMPLIACION_CAPACIDAD

				FROM tb_centrales_d tcd
				INNER JOIN (tb_alimentacion tal 
				INNER JOIN (tb_lineas til 
				INNER JOIN tb_centrales_p tp 
				ON til.central_p=tp.codigo_central)
				ON tal.id_linea=til.codigo_linea)ON tcd.direccion=tal.direccion
				GROUP BY til.codigo_linea, tp.tipo_central
				ORDER BY PORCENTAJE_CARGA_LINEA DESC)
		WHERE ROWNUM BETWEEN 1 AND 10; 

        --Recorremos el cursor para comprobar resultados a nivel particular, el cursor ser tratado por la aplicacin que
	--ha llamado al procedimiento desde donde deber ser cerrado para liberar los recursos
        LOOP FETCH CURSOR_R INTO codi_linea, carga_linea, ampliacion;
        EXIT WHEN CURSOR_R%NOTFOUND;  
		dbms_output.put_line(codi_linea ||'   '||carga_linea||'    '||ampliacion);
        END LOOP;
	        
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,'cursor con los datos solicitados',RSP); 
        COMMIT;
 
     EXCEPTION

        WHEN OTHERS THEN 
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;

