/*Procedimiento que a partir de la lectura del consumo de los contadores  nos permitir determinar
 *el porcentaje de las mismas que se han realizado de manera presencial o de manera telemtica
 * en un period determinado
 *PARAMETROS IN: fecha_inicial(Variable de tipo DATE, contiene la fecha inicial del periodo a consultar)
 *               fecha_final (Variable de tipo DATE, cotiene la fecha final del periodo a consultar)
 *               CURSOR_R (Variable de tipo SYS_REFCURSOR, que se corresponde con el parmetro de retorno que contendr
 *			   los datos solicitados)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: 	 CURSOR_R (Contendr la respuesta con el Cdigo del contrato , la potencia y el porcentaje con respecto al 
 *		 	   consumo medio de los contadores de la Ciudad, segn las especificaciones inciales).
 *		 Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */


CREATE OR REPLACE
PROCEDURE pr_consulta_f(fecha_inicial IN DATE, fecha_final IN DATE, CURSOR_R OUT NOCOPY SYS_REFCURSOR, RSP OUT NOCOPY CLOB) AS

    --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(35);
    In_log CLOB;

    --Declaracin de variables 

    porcentaje_presencial NUMBER(5,2);
    porcentaje_telematica NUMBER(5,2);

    --Variable de comprobacion
    existe_lectura INTEGER;
    
    --Excepciones
    EX_LECTURA_NO_EXISTE EXCEPTION;
   
    --Iniciamos
    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_consulta_f';
      
        --Creamos una cadena de texto con los parametros de entrada.

        In_log:= (TO_CHAR(fecha_inicial,'DD/MM/YYYY')||'  ||  ' || TO_CHAR(fecha_final,'DD/MM/YYYY'));  
        
        --Comprobamos que existen lecturas en el intervalo de fechas pasado por parametro.
        
        SELECT COUNT(*)INTO existe_lectura FROM tb_Lecturas tl 
        WHERE tl.fecha BETWEEN TO_DATE(fecha_inicial) AND TO_DATE( fecha_final);
        
        IF(existe_lectura=0)THEN RAISE EX_LECTURA_NO_EXISTE; END IF;

        
       --Abrimos el cursor
       
        OPEN CURSOR_R FOR
		SELECT ((SELECT COUNT(*) FROM TB_LECTURAS WHERE MODALIDAD='PRESENCIAL'
		        AND FECHA BETWEEN TO_DATE(fecha_inicial) AND TO_DATE(fecha_final))*100)/(existe_lectura)
		        AS PORCENTAJE_PRESENCIAL,
		       ((SELECT COUNT(*) FROM TB_LECTURAS WHERE MODALIDAD='TELEMATICA'
		        AND FECHA BETWEEN TO_DATE(fecha_inicial) AND TO_DATE(fecha_final))*100)/(existe_lectura)
		        AS PORCENTAJE_TELEMATICA
		FROM DUAL;
        
        --Recorremos el cursor para comprobar resultados 
        LOOP FETCH CURSOR_R INTO porcentaje_presencial, porcentaje_telematica;
        EXIT WHEN CURSOR_R%NOTFOUND;  
		dbms_output.put_line(porcentaje_presencial ||'   '||porcentaje_telematica);
        END LOOP;
	        
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,'cursor con los datos solicitados',RSP); 
        COMMIT;
 
     EXCEPTION

        
        WHEN EX_LECTURA_NO_EXISTE THEN
        RSP:='ERROR: PRCOF001: No existen lecturas en el intervalo de tiempo dado';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
        
        WHEN OTHERS THEN 
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;

